@@ -27,6 +27,10 @@
 #include <CL/opencl.h>
 #include "poclu.h"
 
+#ifdef _MSC_VER
+#  include "vccompat.hpp"
+#endif
+
 #define WIDTH 256
 #define HEIGHT 4096
 #define PADDING 32
@@ -78,7 +82,7 @@ main (void)
   for (i = 0; i < HEIGHT; ++i)
     {
       for (j = 0; j < WIDTH; ++j)
-	input[i * WIDTH + j] = drand48();
+      input[i * WIDTH + j] = (cl_float)drand48();
     }
   
   context = poclu_create_any_context();
@@ -87,7 +91,7 @@ main (void)
     return -1; 
 
   clGetContextInfo(context, CL_CONTEXT_DEVICES, 0, NULL, &cb); 
-  devices = (cl_device_id *) malloc(cb); 
+  devices = (cl_device_id *) malloc(cb);
   clGetContextInfo(context, CL_CONTEXT_DEVICES, cb, devices, NULL); 
  
   cmd_queue = clCreateCommandQueue(context, devices[0], 0, NULL); 
