@@ -33,6 +33,8 @@
 
 #include "pocl_types.h"
 
+#include "_kernel_constants.h"
+
 /* Function/type attributes supported by Clang/SPIR */
 #if __has_attribute(__always_inline__)
 #  define _CL_ALWAYSINLINE __attribute__((__always_inline__))
@@ -153,73 +155,29 @@ typedef ulong ulong16 __attribute__((__ext_vector_type__(16)));
 
 /* Starting from Clang 3.3 the image and sampler are detected
    as opaque types by the frontend. In order to define
-   the default builtins we use C functions which require 
-   the typedefs to the actual underlying types. Clang 3.2
-   the typedefs throughout as the types are not detected
-   by the frontend. */
-#if !defined(_CL_HAS_IMAGE_ACCESS)
+   the default builtins we use C functions which require
+   the typedefs to the actual underlying types.
+*/
+#if defined(__CBUILD__)
 typedef int sampler_t;
-typedef struct dev_image_t* image2d_t;
-typedef struct dev_image_t* image3d_t;
-typedef struct dev_image_t* image1d_t;
-typedef struct dev_image_t* image1d_buffer_t;
-typedef struct dev_image_t* image2d_array_t;
-typedef struct dev_image_t* image1d_array_t;
+
+/* Since some built-ins have different return types
+ * (e.g. get_image_dim returns an int2 for 2D images and arrays,
+ *  but an int4 for 3D images) we want each image type to
+ * point to a different type which is actually always the same.
+ * We do this by making it pointer to structs whose only element is a
+ * dev_image_t. The structs are not anonymous to allow identification
+ * by name.
+ */
+typedef struct _pocl_image2d_t { dev_image_t base; }* image2d_t;
+typedef struct _pocl_image3d_t { dev_image_t base; }* image3d_t;
+typedef struct _pocl_image1d_t { dev_image_t base; }* image1d_t;
+typedef struct _pocl_image1d_buffer_t { dev_image_t base; }* image1d_buffer_t;
+typedef struct _pocl_image2d_array_t { dev_image_t base; }* image2d_array_t;
+typedef struct _pocl_image1d_array_t { dev_image_t base; }* image1d_array_t;
 #endif
 
 
-/* cl_channel_order */
-#define CL_R                                        0x10B0
-#define CL_A                                        0x10B1
-#define CL_RG                                       0x10B2
-#define CL_RA                                       0x10B3
-#define CL_RGB                                      0x10B4
-#define CL_RGBA                                     0x10B5
-#define CL_BGRA                                     0x10B6
-#define CL_ARGB                                     0x10B7
-#define CL_INTENSITY                                0x10B8
-#define CL_LUMINANCE                                0x10B9
-#define CL_Rx                                       0x10BA
-#define CL_RGx                                      0x10BB
-#define CL_RGBx                                     0x10BC
-#define CL_DEPTH                                    0x10BD
-#define CL_DEPTH_STENCIL                            0x10BE
-
-/* cl_channel_type */
-#define CL_SNORM_INT8                               0x10D0
-#define CL_SNORM_INT16                              0x10D1
-#define CL_UNORM_INT8                               0x10D2
-#define CL_UNORM_INT16                              0x10D3
-#define CL_UNORM_SHORT_565                          0x10D4
-#define CL_UNORM_SHORT_555                          0x10D5
-#define CL_UNORM_INT_101010                         0x10D6
-#define CL_SIGNED_INT8                              0x10D7
-#define CL_SIGNED_INT16                             0x10D8
-#define CL_SIGNED_INT32                             0x10D9
-#define CL_UNSIGNED_INT8                            0x10DA
-#define CL_UNSIGNED_INT16                           0x10DB
-#define CL_UNSIGNED_INT32                           0x10DC
-#define CL_HALF_FLOAT                               0x10DD
-#define CL_FLOAT                                    0x10DE
-#define CL_UNORM_INT24                              0x10DF
-
-/* cl_addressing _mode */
-#define CLK_ADDRESS_NONE                            0x00
-#define CLK_ADDRESS_MIRRORED_REPEAT                 0x01
-#define CLK_ADDRESS_REPEAT                          0x02
-#define CLK_ADDRESS_CLAMP_TO_EDGE                   0x03
-#define CLK_ADDRESS_CLAMP                           0x04
-
-/* cl_sampler_info */
-#define CLK_NORMALIZED_COORDS_FALSE                 0x00
-#define CLK_NORMALIZED_COORDS_TRUE                  0x08
-
-/* filter_mode */
-#define CLK_FILTER_NEAREST                          0x00
-#define CLK_FILTER_LINEAR                           0x10
-
-//#ifdef _CL_HAS_IMAGE_ACCESS
-
 float4 _CL_OVERLOADABLE read_imagef (image2d_t image, sampler_t sampler,
                                      int2 coord);
 
@@ -298,4 +256,9 @@ int _CL_OVERLOADABLE get_image_height (image3d_t image);
 int _CL_OVERLOADABLE get_image_depth (image1d_t image);
 int _CL_OVERLOADABLE get_image_depth (image2d_t image);
 int _CL_OVERLOADABLE get_image_depth (image3d_t image);
+
+int2 _CL_OVERLOADABLE get_image_dim (image2d_t image);
+int2 _CL_OVERLOADABLE get_image_dim (image2d_array_t image);
+int4 _CL_OVERLOADABLE get_image_dim (image3d_t image);
+
 #endif
