@@ -36,6 +36,7 @@
 #include <CL/opencl.h>
 
 #include "pocl_device.h"
+#include "config.h"
 
 /*
  * During pocl kernel compiler transformations we use the fixed address 
@@ -48,6 +49,9 @@
 #define POCL_ADDRESS_SPACE_GLOBAL 1
 #define POCL_ADDRESS_SPACE_LOCAL 2
 #define POCL_ADDRESS_SPACE_CONSTANT 3
+#define POCL_ADDRESS_SPACE_GENERIC 4
+
+#define POCL_FILENAME_LENGTH 1024
 
 typedef struct _mem_mapping mem_mapping_t;
 /* represents a single buffer to host memory mapping */
@@ -70,12 +74,14 @@ typedef struct
   /* A list of argument buffers to free after the command has 
      been executed. */
   cl_mem *arg_buffers;
-  int arg_buffer_count;
+  unsigned arg_buffer_count;
   size_t local_x;
   size_t local_y;
   size_t local_z;
   struct pocl_context pc;
   struct pocl_argument *arguments;
+  /* Can be used to store/cache device-specific data. */
+  void **device_data;
 } _cl_command_run;
 
 // clEnqueueNativeKernel
@@ -86,7 +92,7 @@ typedef struct
   size_t cb_args;
   void (*user_func)(void *);
   cl_mem *mem_list;
-  int num_mem_objects;
+  unsigned num_mem_objects;
 } _cl_command_native;
 
 // clEnqueueReadBuffer
@@ -94,6 +100,7 @@ typedef struct
 {
   void *host_ptr;
   const void *device_ptr;
+  size_t offset;
   size_t cb;
   cl_mem buffer;
 } _cl_command_read;
@@ -103,6 +110,7 @@ typedef struct
 {
   const void *host_ptr;
   void *device_ptr;
+  size_t offset;
   size_t cb;
   cl_mem buffer;
 } _cl_command_write;
@@ -112,7 +120,9 @@ typedef struct
 {
   void *data;
   void *src_ptr;
+  size_t src_offset;
   void *dst_ptr;
+  size_t dst_offset;
   size_t cb;
   cl_mem src_buffer;
   cl_mem dst_buffer;
@@ -125,7 +135,6 @@ typedef struct
   mem_mapping_t *mapping;
 } _cl_command_map;
 
-
 /* clEnqueue(Write/Read)Image */
 typedef struct
 {
@@ -159,11 +168,50 @@ typedef struct
   size_t pixel_size;
 } _cl_command_fill_image;
 
+typedef struct
+{
+  void* ptr;
+  size_t size, offset;
+  void* pattern;
+  size_t pattern_size;
+} _cl_command_fill;
+
 typedef struct
 {
   void *data;
 } _cl_command_marker;
 
+typedef struct
+{
+  void* data;
+  void* queue;
+  unsigned  num_svm_pointers;
+  void  **svm_pointers;
+  void (CL_CALLBACK  *pfn_free_func) ( cl_command_queue queue,
+                                       unsigned num_svm_pointers,
+                                       void *svm_pointers[],
+                                       void  *user_data);
+} _cl_command_svm_free;
+
+typedef struct
+{
+  void* svm_ptr;
+  size_t size;
+  cl_map_flags flags;
+} _cl_command_svm_map;
+
+typedef struct
+{
+  void* svm_ptr;
+} _cl_command_svm_unmap;
+
+typedef struct
+{
+  const void* src;
+  void* dst;
+  size_t size;
+} _cl_command_svm_cpy;
+
 typedef union
 {
   _cl_command_run run;
@@ -176,18 +224,30 @@ typedef union
   _cl_command_rw_image rw_image;
   _cl_command_marker marker;
   _cl_command_unmap unmap;
+  _cl_command_fill memfill;
+
+  _cl_command_svm_free svm_free;
+  _cl_command_svm_map svm_map;
+  _cl_command_svm_unmap svm_unmap;
+  _cl_command_svm_cpy svm_memcpy;
 } _cl_command_t;
 
 // one item in the command queue
-typedef struct
+typedef struct _cl_command_node_struct
 {
   _cl_command_t command;
   cl_command_type type;
-  void *next; // for linked-list storage
+  struct _cl_command_node_struct *next; // for linked-list storage
   cl_event event;
   const cl_event *event_wait_list;
-  cl_int num_events_in_wait_list;
+  cl_uint num_events_in_wait_list;
   cl_device_id device;
 } _cl_command_node;
 
+/* Additional LLVM version macros to simplify ifdefs */
+
+#if (defined LLVM_3_7)
+# define LLVM_OLDER_THAN_3_8 1
+#endif
+
 #endif /* POCL_H */
