@@ -22,7 +22,6 @@
    THE SOFTWARE.
 */
 
-#include "pocl_cl.h"
 #include "utlist.h"
 #include <assert.h>
 #include "pocl_util.h"
@@ -33,39 +32,47 @@ POname(clEnqueueCopyBuffer)(cl_command_queue command_queue,
                             cl_mem dst_buffer,
                             size_t src_offset,
                             size_t dst_offset,
-                            size_t cb, 
+                            size_t size,
                             cl_uint num_events_in_wait_list,
                             const cl_event *event_wait_list,
                             cl_event *event) 
 CL_API_SUFFIX__VERSION_1_0
 {
-  cl_device_id device_id;
+  cl_device_id device;
   unsigned i;
   _cl_command_node *cmd = NULL;
   int errcode;
 
-  if (command_queue == NULL)
-    return CL_INVALID_COMMAND_QUEUE;
+  POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
-  if ((src_buffer == NULL) || (dst_buffer == NULL))
-    return CL_INVALID_MEM_OBJECT;
+  POCL_RETURN_ERROR_COND((src_buffer == NULL), CL_INVALID_MEM_OBJECT);
 
-  if ((command_queue->context != src_buffer->context) ||
-      (command_queue->context != dst_buffer->context))
-    return CL_INVALID_CONTEXT;
+  POCL_RETURN_ERROR_COND((dst_buffer == NULL), CL_INVALID_MEM_OBJECT);
 
-  if ((src_offset + cb > src_buffer->size) ||
-      (dst_offset + cb > dst_buffer->size))
-    return CL_INVALID_VALUE;
+  POCL_RETURN_ERROR_ON((src_buffer->type != CL_MEM_OBJECT_BUFFER),
+      CL_INVALID_MEM_OBJECT, "src_buffer is not a CL_MEM_OBJECT_BUFFER\n");
+  POCL_RETURN_ERROR_ON((dst_buffer->type != CL_MEM_OBJECT_BUFFER),
+      CL_INVALID_MEM_OBJECT, "dst_buffer is not a CL_MEM_OBJECT_BUFFER\n");
 
-  device_id = command_queue->device;
+  POCL_RETURN_ERROR_ON(((command_queue->context != src_buffer->context) ||
+      (command_queue->context != dst_buffer->context)), CL_INVALID_CONTEXT,
+      "src_buffer, dst_buffer and command_queue are not from the same context\n");
 
-  for (i = 0; i < command_queue->context->num_devices; ++i)
-    {
-      if (command_queue->context->devices[i] == device_id)
-        break;
-    }
-  assert(i < command_queue->context->num_devices);
+  POCL_RETURN_ERROR_COND((size == 0), CL_INVALID_VALUE);
+
+  POCL_RETURN_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  POCL_RETURN_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  if (pocl_buffers_boundcheck(src_buffer, dst_buffer, src_offset,
+        dst_offset, size) != CL_SUCCESS) return CL_INVALID_VALUE;
+
+  if (pocl_buffers_overlap(src_buffer, dst_buffer, src_offset,
+        dst_offset, size) != CL_SUCCESS) return CL_MEM_COPY_OVERLAP;
+
+  POCL_CHECK_DEV_IN_CMDQ;
 
   errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_COPY_BUFFER, 
                                  event, num_events_in_wait_list, 
@@ -79,14 +86,17 @@ CL_API_SUFFIX__VERSION_1_0
   POname(clRetainMemObject)(src_buffer);
   POname(clRetainMemObject)(dst_buffer);
 
-  cmd->command.copy.data = device_id->data;
+  cmd->command.copy.data = device->data;
   /* TODO: call device->buf_offset() or similar as device_ptrs might not be
      actual buffer pointers but pointers to a book keeping structure. */
   cmd->command.copy.src_ptr = 
-    src_buffer->device_ptrs[device_id->dev_id].mem_ptr + src_offset;
+    (char*)src_buffer->device_ptrs[device->dev_id].mem_ptr;
+  cmd->command.copy.src_offset = src_offset;
+
   cmd->command.copy.dst_ptr = 
-    dst_buffer->device_ptrs[device_id->dev_id].mem_ptr + dst_offset;
-  cmd->command.copy.cb = cb;
+    (char*)dst_buffer->device_ptrs[device->dev_id].mem_ptr;
+  cmd->command.copy.dst_offset = dst_offset;
+  cmd->command.copy.cb = size;
 
   pocl_command_enqueue(command_queue, cmd);
 
