@@ -39,58 +39,55 @@ POname(clEnqueueMapImage)(cl_command_queue   command_queue,
                           size_t *           image_slice_pitch,
                           cl_uint            num_events_in_wait_list,
                           const cl_event *   event_wait_list,
-                          cl_event *         event,
-                          cl_int *           errcode_ret ) 
+                          cl_event *         event_ret,
+                          cl_int *           errcode_ret )
 CL_API_SUFFIX__VERSION_1_0
 {
   cl_int errcode;
-  int offset;
+  size_t offset;
   void *map = NULL;
   cl_device_id device;
+  cl_event event = NULL;
   _cl_command_node *cmd = NULL;
   mem_mapping_t *mapping_info = NULL;
 
+  POCL_GOTO_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
+
   device = command_queue->device;
-  if (command_queue == NULL)
-    {
-      errcode =  CL_INVALID_COMMAND_QUEUE;
-      goto ERROR;
-    }
-  if (image == NULL)
-    {
-      errcode = CL_INVALID_MEM_OBJECT;
-      goto ERROR;
-    }
-  if (command_queue->context != image->context)
-    {
-      errcode = CL_INVALID_CONTEXT;
-      goto ERROR;
-    }
-  if ((event_wait_list == NULL && num_events_in_wait_list != 0) ||
-      (event_wait_list != NULL && num_events_in_wait_list == 0))
-    {
-      errcode = CL_INVALID_EVENT_WAIT_LIST;
-      goto ERROR;
-    }
+
+  POCL_GOTO_ERROR_ON((!command_queue->device->image_support), CL_INVALID_OPERATION,
+    "Device %s does not support images\n", command_queue->device->long_name);
+
+  POCL_GOTO_ERROR_ON((command_queue->context != image->context),
+    CL_INVALID_CONTEXT, "image and command_queue are not from the same context\n");
+
+  POCL_GOTO_ERROR_COND((image == NULL), CL_INVALID_MEM_OBJECT);
+
+  POCL_GOTO_ERROR_ON((!image->is_image), CL_INVALID_MEM_OBJECT,
+    "image argument is not an image type cl_mem\n");
+
+  POCL_GOTO_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  POCL_GOTO_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  errcode = pocl_check_device_supports_image(image, command_queue);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
+
+  POCL_GOTO_ERROR_COND((image_row_pitch == NULL), CL_INVALID_VALUE);
 
   errcode = pocl_check_image_origin_region(image, origin, region);
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
-  if (image_row_pitch == NULL)
-    {
-      errcode = CL_INVALID_VALUE;
-      goto ERROR;
-    }
-
-  if (image_slice_pitch == NULL && 
+  POCL_GOTO_ERROR_ON((image_slice_pitch == NULL &&
       (image->type == CL_MEM_OBJECT_IMAGE3D || 
        image->type == CL_MEM_OBJECT_IMAGE1D_ARRAY ||
-       image->type == CL_MEM_OBJECT_IMAGE2D_ARRAY))
-    {
-      errcode = CL_INVALID_VALUE;
-      goto ERROR;
-    }
+       image->type == CL_MEM_OBJECT_IMAGE2D_ARRAY)), CL_INVALID_VALUE,
+       "For a 3D image, 1D, and 2D image array, "
+       "image_slice_pitch must be a non-NULL value\n");
 
   /* TODO: more error checks */
   
@@ -103,18 +100,24 @@ CL_API_SUFFIX__VERSION_1_0
       goto ERROR;
     }
 
+  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MAP_IMAGE,
+                                 &event, num_events_in_wait_list,
+                                 event_wait_list);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
+
   if (image->flags & CL_MEM_USE_HOST_PTR)
     {
       /* In this case it should use the given host_ptr + offset as
-         the mapping area in the host memory. */   
+         the mapping area in the host memory. */
       assert (image->mem_host_ptr != NULL);
-      map = image->mem_host_ptr + offset;
-      device->ops->read_rect (device->data, map, 
+      map = (char*)image->mem_host_ptr + offset;
+      device->ops->read_rect (device->data, map,
                               image->device_ptrs[device->dev_id].mem_ptr,
-                              origin, origin, region, 
-                              image->image_row_pitch, image->image_slice_pitch, 
+                              origin, origin, region,
+                              image->image_row_pitch, image->image_slice_pitch,
                               image->image_row_pitch, image->image_slice_pitch);
-      POCL_UPDATE_EVENT_COMPLETE(event, command_queue);
+      POCL_UPDATE_EVENT_COMPLETE(&event);
     }
   else
     {
@@ -131,7 +134,6 @@ CL_API_SUFFIX__VERSION_1_0
 
   if (map == NULL)
     {
-      POCL_UPDATE_EVENT_COMPLETE(event, command_queue);
       errcode = CL_MAP_FAILURE;
       goto ERROR;
     }
@@ -143,40 +145,34 @@ CL_API_SUFFIX__VERSION_1_0
   DL_APPEND (image->mappings, mapping_info);
   POCL_UNLOCK_OBJ (image);
 
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MAP_IMAGE, 
-                                 event, num_events_in_wait_list, 
-                                 event_wait_list);
-  if (errcode != CL_SUCCESS)
-    goto ERROR;
-      
-  
   cmd->command.map.buffer = image;
   cmd->command.map.mapping = mapping_info;
   pocl_command_enqueue(command_queue, cmd);
-  
-  if (blocking_map)
-    {
-      POname(clFinish) (command_queue);
-    }
-  
+
   *image_row_pitch = image->image_row_pitch;
   if (image_slice_pitch)
     *image_slice_pitch = image->image_slice_pitch;
 
+  if (blocking_map)
+    {
+      POname(clFinish) (command_queue);
+    }
+
+  if (event_ret)
+    *event_ret = event;
   if (errcode_ret != NULL)
     (*errcode_ret) = CL_SUCCESS;
 
   return map;
- 
- ERROR:
-  free (map);
-  free (cmd);
-  free (mapping_info);
-  if (event != NULL)
-    free (*event);
+
+ERROR:
+  POCL_MEM_FREE(event);
+  assert(map == NULL);
+  POCL_MEM_FREE(cmd);
+  POCL_MEM_FREE(mapping_info);
   if(errcode_ret != NULL)
     (*errcode_ret) = errcode;
-      
+
   return NULL;
 }
 POsym(clEnqueueMapImage)
