@@ -35,20 +35,23 @@ POname(clEnqueueUnmapMemObject)(cl_command_queue command_queue,
                         cl_event *       event) CL_API_SUFFIX__VERSION_1_0
 {
   int errcode;
-  cl_device_id device_id;
+  cl_device_id device;
   unsigned i;
   mem_mapping_t *mapping = NULL;
-  _cl_command_node *cmd;
+  _cl_command_node *cmd = NULL;
 
-  if (memobj == NULL)
-    return CL_INVALID_MEM_OBJECT;
+  POCL_RETURN_ERROR_COND((memobj == NULL), CL_INVALID_MEM_OBJECT);
 
-  if (command_queue == NULL || command_queue->device == NULL ||
-      command_queue->context == NULL)
-    return CL_INVALID_COMMAND_QUEUE;
+  POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
-  if (command_queue->context != memobj->context)
-    return CL_INVALID_CONTEXT;
+  POCL_RETURN_ERROR_ON((command_queue->context != memobj->context),
+    CL_INVALID_CONTEXT, "memobj and command_queue are not from the same context\n");
+
+  POCL_RETURN_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  POCL_RETURN_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
+    CL_INVALID_EVENT_WAIT_LIST);
 
   POCL_LOCK_OBJ (memobj);
   DL_FOREACH (memobj->mappings, mapping)
@@ -57,26 +60,19 @@ POname(clEnqueueUnmapMemObject)(cl_command_queue command_queue,
           break;
     }
   POCL_UNLOCK_OBJ (memobj);
-  if (mapping == NULL)
-    return CL_INVALID_VALUE;
+  POCL_RETURN_ERROR_ON((mapping == NULL), CL_INVALID_VALUE,
+      "Could not find mapping of this memobj\n");
 
   /* find the index of the device's ptr in the buffer */
-  device_id = command_queue->device;
-  for (i = 0; i < command_queue->context->num_devices; ++i)
-    {
-      if (command_queue->context->devices[i] == device_id)
-        break;
-    }
+  POCL_CHECK_DEV_IN_CMDQ;
 
-  assert(i < command_queue->context->num_devices);
-
-  errcode = pocl_create_command (&cmd, command_queue, 
-                                 CL_COMMAND_UNMAP_MEM_OBJECT, 
-                                 event, num_events_in_wait_list, 
+  errcode = pocl_create_command (&cmd, command_queue,
+                                 CL_COMMAND_UNMAP_MEM_OBJECT,
+                                 event, num_events_in_wait_list,
                                  event_wait_list);
   if (errcode != CL_SUCCESS)
     goto ERROR;
-  
+
   cmd->command.unmap.data = command_queue->device->data;
   cmd->command.unmap.memobj = memobj;
   cmd->command.unmap.mapping = mapping;
@@ -84,9 +80,8 @@ POname(clEnqueueUnmapMemObject)(cl_command_queue command_queue,
 
   return CL_SUCCESS;
 
- ERROR:
-  free (*event);
-  free (cmd);
+ERROR:
+  POCL_MEM_FREE(cmd);
   return errcode;
 }
 POsym(clEnqueueUnmapMemObject)
