@@ -5,7 +5,7 @@ POname(clEnqueueWaitForEvents)(cl_command_queue  command_queue,
                        const cl_event *  event_list) 
 CL_API_SUFFIX__VERSION_1_0
 {
-  POCL_ABORT_UNIMPLEMENTED();
+  POCL_ABORT_UNIMPLEMENTED("The entire clEnqueueWaitForEvents call");
   return CL_SUCCESS;
 }
 POsym(clEnqueueWaitForEvents)
