@@ -43,33 +43,45 @@ POname(clEnqueueWriteBufferRect)(cl_command_queue command_queue,
 {
   cl_device_id device;
   unsigned i;
-  int errcode;
 
-  if (command_queue == NULL)
-    return CL_INVALID_COMMAND_QUEUE;
+  POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
-  if (buffer == NULL)
-    return CL_INVALID_MEM_OBJECT;
+  POCL_RETURN_ERROR_COND((buffer == NULL), CL_INVALID_MEM_OBJECT);
 
-  if (command_queue->context != buffer->context)
-    return CL_INVALID_CONTEXT;
+  POCL_RETURN_ERROR_ON((buffer->type != CL_MEM_OBJECT_BUFFER),
+      CL_INVALID_MEM_OBJECT, "buffer is not a CL_MEM_OBJECT_BUFFER\n");
 
-  if ((ptr == NULL) ||
-      (buffer_origin == NULL) ||
-      (host_origin == NULL) ||
-      (region == NULL))
-    return CL_INVALID_VALUE;
+  POCL_RETURN_ERROR_ON((buffer->flags & (CL_MEM_HOST_READ_ONLY | CL_MEM_HOST_NO_ACCESS)),
+    CL_INVALID_OPERATION, "buffer has been created with CL_MEM_HOST_READ_ONLY "
+    "or CL_MEM_HOST_NO_ACCESS\n");
 
-  if ((region[0]*region[1]*region[2] > 0) &&
-      (buffer_origin[0] + region[0]-1 +
-       buffer_row_pitch * (buffer_origin[1] + region[1]-1) +
-       buffer_slice_pitch * (buffer_origin[2] + region[2]-1) >= buffer->size))
-  {
-    POCL_ABORT_UNIMPLEMENTED();
-    return CL_INVALID_VALUE;
-  }
+  POCL_RETURN_ERROR_ON((command_queue->context != buffer->context),
+    CL_INVALID_CONTEXT, "buffer and command_queue are not from the same context\n");
 
-  device = command_queue->device;
+  POCL_RETURN_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  POCL_RETURN_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
+    CL_INVALID_EVENT_WAIT_LIST);
+
+  POCL_RETURN_ERROR_COND((ptr == NULL), CL_INVALID_VALUE);
+  POCL_RETURN_ERROR_COND((buffer_origin == NULL), CL_INVALID_VALUE);
+  POCL_RETURN_ERROR_COND((host_origin == NULL), CL_INVALID_VALUE);
+  POCL_RETURN_ERROR_COND((region == NULL), CL_INVALID_VALUE);
+
+  size_t region_bytes = region[0] * region[1] * region[2];
+  POCL_RETURN_ERROR_ON((region_bytes <= 0), CL_INVALID_VALUE, "All items in region must be >0\n");
+
+  if (pocl_buffer_boundcheck_3d(buffer->size, buffer_origin, region, &buffer_row_pitch,
+      &buffer_slice_pitch, "") != CL_SUCCESS) return CL_INVALID_VALUE;
+
+  if (pocl_buffer_boundcheck_3d(((size_t)-1), host_origin, region, &host_row_pitch,
+      &host_slice_pitch, "") != CL_SUCCESS) return CL_INVALID_VALUE;
+
+
+
+
+  device = POCL_REAL_DEV(command_queue->device);
 
   for (i = 0; i < command_queue->context->num_devices; ++i)
     {
@@ -85,7 +97,7 @@ POname(clEnqueueWriteBufferRect)(cl_command_queue command_queue,
   if (command_queue->properties & CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE)
     {
       /* wait for the event in event_wait_list to finish */
-      POCL_ABORT_UNIMPLEMENTED();
+      POCL_ABORT_UNIMPLEMENTED("clEnqueueReadBufferRect: Out-of-order queue");
     }
   else
     {
@@ -96,7 +108,7 @@ POname(clEnqueueWriteBufferRect)(cl_command_queue command_queue,
       POname(clFinish)(command_queue);
     }
 
-  POCL_UPDATE_EVENT_RUNNING(event, command_queue);
+  POCL_UPDATE_EVENT_RUNNING(event);
 
   /* TODO: offset computation doesn't work in case the ptr is not 
      a direct pointer */
@@ -106,7 +118,7 @@ POname(clEnqueueWriteBufferRect)(cl_command_queue command_queue,
                            buffer_row_pitch, buffer_slice_pitch,
                            host_row_pitch, host_slice_pitch);
 
-  POCL_UPDATE_EVENT_COMPLETE(event, command_queue);
+  POCL_UPDATE_EVENT_COMPLETE(event);
 
   POname(clReleaseMemObject) (buffer);
 
