@@ -21,9 +21,7 @@
    THE SOFTWARE.
 */
 
-#include "pocl_cl.h"
 #include "pocl_util.h"
-#include <string.h>
 
 /* A version for querying the info and in case the device returns 
    a zero, assume the device info query hasn't been implemented 
@@ -43,18 +41,6 @@
     return CL_SUCCESS;                                              \
   } 
 
-#define POCL_RETURN_DEVICE_INFO_STR(__STR__)                        \
-  {                                                                 \
-    size_t const value_size = strlen(__STR__) + 1;                  \
-    if (param_value)                                                \
-      {                                                             \
-        if (param_value_size < value_size) return CL_INVALID_VALUE; \
-        memcpy(param_value, __STR__, value_size);                   \
-      }                                                             \
-    if (param_value_size_ret)                                       \
-      *param_value_size_ret = value_size;                           \
-    return CL_SUCCESS;                                              \
-  }                                                                 \
     
 
   
@@ -194,23 +180,21 @@ POname(clGetDeviceInfo)(cl_device_id   device,
     POCL_RETURN_GETINFO(cl_bool, device->compiler_available);
   case CL_DEVICE_EXECUTION_CAPABILITIES            :
     POCL_RETURN_GETINFO(cl_device_exec_capabilities, device->execution_capabilities);
-  case CL_DEVICE_QUEUE_PROPERTIES                  :
-    POCL_RETURN_GETINFO(cl_command_queue_properties, device->queue_properties);
    
   case CL_DEVICE_NAME:
-    POCL_RETURN_DEVICE_INFO_STR(device->long_name);
+    POCL_RETURN_GETINFO_STR(device->long_name);
    
   case CL_DEVICE_VENDOR                            : 
-    POCL_RETURN_DEVICE_INFO_STR(device->vendor);
+    POCL_RETURN_GETINFO_STR(device->vendor);
 
   case CL_DRIVER_VERSION:
-    POCL_RETURN_DEVICE_INFO_STR(device->driver_version);
+    POCL_RETURN_GETINFO_STR(device->driver_version);
   case CL_DEVICE_PROFILE                           : 
-    POCL_RETURN_DEVICE_INFO_STR(device->profile);
+    POCL_RETURN_GETINFO_STR(device->profile);
   case CL_DEVICE_VERSION                           : 
-    POCL_RETURN_DEVICE_INFO_STR(device->version); 
+    POCL_RETURN_GETINFO_STR(device->version);
   case CL_DEVICE_EXTENSIONS                        : 
-    POCL_RETURN_DEVICE_INFO_STR(device->extensions);
+    POCL_RETURN_GETINFO_STR(device->extensions);
   case CL_DEVICE_PLATFORM                          :
     {
       /* Return the first platform id, assuming this is the only
@@ -242,26 +226,23 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF          : 
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->native_vector_width_half);
   case CL_DEVICE_OPENCL_C_VERSION                  :
-    POCL_RETURN_DEVICE_INFO_STR("OpenCL C 1.2");
+    POCL_RETURN_GETINFO_STR("OpenCL C 2.0");
   case CL_DEVICE_BUILT_IN_KERNELS                  :
-    POCL_RETURN_DEVICE_INFO_STR("");
+    POCL_RETURN_GETINFO_STR("");
 
   /* TODO proper device partition support. For the time being,
    * the values returned only serve the purpose of indicating
    * that it is not actually supported */
   case CL_DEVICE_PARENT_DEVICE                     :
-    POCL_RETURN_GETINFO(cl_device_id, NULL);
+    POCL_RETURN_GETINFO(cl_device_id, device->parent_device);
   case CL_DEVICE_PARTITION_MAX_SUB_DEVICES         :
-    POCL_RETURN_GETINFO(cl_uint, 1);
+    POCL_RETURN_GETINFO(cl_uint, device->max_sub_devices);
   case CL_DEVICE_PARTITION_PROPERTIES              :
+    POCL_RETURN_GETINFO_ARRAY(cl_device_partition_property,
+      device->num_partition_properties, device->partition_properties);
   case CL_DEVICE_PARTITION_TYPE                    :
-    {
-      /* since we don't support sub-devices, querying the partition type
-       * presently returns the same thing as querying the available partition
-       * properties, i.e. { 0} */
-      typedef struct { cl_device_partition_property prop[1]; } dev_pp_1;
-      POCL_RETURN_GETINFO(dev_pp_1, *(const dev_pp_1*)device->device_partition_properties);
-    }
+    POCL_RETURN_GETINFO_ARRAY(cl_device_partition_property,
+      device->num_partition_types, device->partition_type);
   case CL_DEVICE_PARTITION_AFFINITY_DOMAIN         :
     POCL_RETURN_GETINFO(cl_device_affinity_domain, 0);
 
@@ -272,6 +253,43 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_REFERENCE_COUNT:
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, 
                                             (cl_uint)device->pocl_refcount)
+
+
+
+
+  case CL_DEVICE_SVM_CAPABILITIES:
+    POCL_RETURN_GETINFO(cl_device_svm_capabilities, device->svm_caps);
+  case CL_DEVICE_MAX_ON_DEVICE_EVENTS:
+    POCL_RETURN_GETINFO(cl_uint, device->max_events);
+  case CL_DEVICE_MAX_ON_DEVICE_QUEUES:
+    POCL_RETURN_GETINFO(cl_uint, device->max_queues);
+  case CL_DEVICE_MAX_PIPE_ARGS:
+    POCL_RETURN_GETINFO(cl_uint, device->max_pipe_args);
+  case CL_DEVICE_PIPE_MAX_ACTIVE_RESERVATIONS:
+    POCL_RETURN_GETINFO(cl_uint, device->max_pipe_active_res);
+  case CL_DEVICE_PIPE_MAX_PACKET_SIZE:
+    POCL_RETURN_GETINFO(cl_uint, device->max_pipe_packet_size);
+  case CL_DEVICE_QUEUE_ON_DEVICE_PREFERRED_SIZE:
+    POCL_RETURN_GETINFO(cl_uint, device->dev_queue_pref_size);
+  case CL_DEVICE_QUEUE_ON_DEVICE_MAX_SIZE:
+    POCL_RETURN_GETINFO(cl_uint, device->dev_queue_max_size);
+  case CL_DEVICE_PREFERRED_GLOBAL_ATOMIC_ALIGNMENT:
+    POCL_RETURN_GETINFO(cl_uint, 0);
+  case CL_DEVICE_PREFERRED_LOCAL_ATOMIC_ALIGNMENT:
+    POCL_RETURN_GETINFO(cl_uint, 0);
+  case CL_DEVICE_PREFERRED_PLATFORM_ATOMIC_ALIGNMENT:
+    POCL_RETURN_GETINFO(cl_uint, 0);
+  case CL_DEVICE_SPIR_VERSIONS:
+    POCL_RETURN_GETINFO_STR("1.2");
+  case CL_DEVICE_QUEUE_ON_DEVICE_PROPERTIES:
+    POCL_RETURN_GETINFO(cl_command_queue_properties, device->on_dev_queue_props);
+  case CL_DEVICE_QUEUE_ON_HOST_PROPERTIES:
+    POCL_RETURN_GETINFO(cl_command_queue_properties, device->on_host_queue_props);
+
+  case CL_DEVICE_GLOBAL_VARIABLE_PREFERRED_TOTAL_SIZE:
+    POCL_RETURN_GETINFO(size_t, device->global_var_pref_size);
+  case CL_DEVICE_MAX_GLOBAL_VARIABLE_SIZE:
+    POCL_RETURN_GETINFO(size_t, device->global_var_max_size);
   }
   return CL_INVALID_VALUE;
 }
