@@ -27,8 +27,12 @@ CL_API_ENTRY cl_int CL_API_CALL
 POname(clReleaseContext)(cl_context context) CL_API_SUFFIX__VERSION_1_0
 {
   int new_refcount;
+  // if context is invalid, return immediately
   if (!context->valid)
-    return CL_INVALID_CONTEXT;
+    {
+      POCL_MEM_FREE(context);
+      return CL_SUCCESS;
+    }
 
   POCL_RELEASE_OBJECT(context, new_refcount);
   if (new_refcount == 0)
@@ -38,14 +42,14 @@ POname(clReleaseContext)(cl_context context) CL_API_SUFFIX__VERSION_1_0
          references and let the objects to get freed. */
       /* TODO: call the corresponding clRelease* functions
          for all the referred objects. */
-      int i;
+      unsigned i;
       for (i = 0; i < context->num_devices; ++i) 
         {
           POname(clReleaseDevice) (context->devices[i]);
         }   
-      free(context->devices);
-      if (context->properties) free(context->properties);
-      free(context);
+      POCL_MEM_FREE(context->devices);
+      POCL_MEM_FREE(context->properties);
+      POCL_MEM_FREE(context);
     }
   return CL_SUCCESS;
 }
