@@ -29,7 +29,10 @@ POname(clReleaseKernel)(cl_kernel kernel) CL_API_SUFFIX__VERSION_1_0
 {
   int new_refcount;
   cl_kernel *pk;
-  int i;
+  unsigned i;
+
+  POCL_RETURN_ERROR_COND((kernel == NULL), CL_INVALID_KERNEL);
+
   POCL_RELEASE_OBJECT (kernel, new_refcount);
 
   if (new_refcount == 0)
@@ -55,8 +58,7 @@ POname(clReleaseKernel)(cl_kernel kernel) CL_API_SUFFIX__VERSION_1_0
           POname(clReleaseProgram) (kernel->program);
         }
       
-      free ((char*)kernel->function_name);
-      free ((char*)kernel->name);
+      POCL_MEM_FREE(kernel->name);
 
       for (i = 0; i < kernel->num_args; i++)
         {
@@ -64,12 +66,13 @@ POname(clReleaseKernel)(cl_kernel kernel) CL_API_SUFFIX__VERSION_1_0
           if (p->value != NULL)
             {
               pocl_aligned_free (p->value);
+              p->value = NULL;
             }
         }
 
-      free (kernel->dyn_arguments);
-      free (kernel->reqd_wg_size);
-      free (kernel);
+      POCL_MEM_FREE(kernel->dyn_arguments);
+      POCL_MEM_FREE(kernel->reqd_wg_size);
+      POCL_MEM_FREE(kernel);
     }
   
   return CL_SUCCESS;
