@@ -22,6 +22,7 @@
    THE SOFTWARE.
 */
 
+#include "config.h"
 #include "basic.h"
 #include "cpuinfo.h"
 #include "topology/pocl_topology.h"
@@ -32,14 +33,15 @@
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <dev_image.h>
-#include <sys/time.h>
+
+#include "pocl_cache.h"
+#include "pocl_timing.h"
+#include "pocl_llvm.h"
 
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
 #define COMMAND_LENGTH 2048
-#define WORKGROUP_STRING_LENGTH 128
+#define WORKGROUP_STRING_LENGTH 1024
 
 struct data {
   /* Currently loaded kernel. */
@@ -48,7 +50,7 @@ struct data {
   lt_dlhandle current_dlhandle;
 };
 
-const cl_image_format supported_image_formats[] = {
+static const cl_image_format supported_image_formats[] = {
     { CL_R, CL_SNORM_INT8 },
     { CL_R, CL_SNORM_INT16 },
     { CL_R, CL_UNORM_INT8 },
@@ -194,6 +196,7 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->init = pocl_basic_init;
   ops->alloc_mem_obj = pocl_basic_alloc_mem_obj;
   ops->free = pocl_basic_free;
+  ops->free_ptr = pocl_basic_free_ptr;
   ops->read = pocl_basic_read;
   ops->read_rect = pocl_basic_read_rect;
   ops->write = pocl_basic_write;
@@ -201,7 +204,9 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->copy = pocl_basic_copy;
   ops->copy_rect = pocl_basic_copy_rect;
   ops->fill_rect = pocl_basic_fill_rect;
+  ops->memfill = pocl_basic_memfill;
   ops->map_mem = pocl_basic_map_mem;
+  ops->unmap_mem = pocl_basic_unmap_mem;
   ops->compile_submitted_kernels = pocl_basic_compile_submitted_kernels;
   ops->run = pocl_basic_run;
   ops->run_native = pocl_basic_run_native;
@@ -216,10 +221,18 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->vendor_id = 0;
   dev->max_compute_units = 0;
   dev->max_work_item_dimensions = 3;
-  dev->max_work_item_sizes[0] = CL_INT_MAX;
-  dev->max_work_item_sizes[1] = CL_INT_MAX;
-  dev->max_work_item_sizes[2] = CL_INT_MAX;
-  dev->max_work_group_size = 1024;
+
+  SETUP_DEVICE_CL_VERSION(HOST_DEVICE_CL_VERSION_MAJOR, HOST_DEVICE_CL_VERSION_MINOR)
+  /*
+    The hard restriction will be the context data which is
+    stored in stack that can be as small as 8K in Linux.
+    Thus, there should be enough work-items alive to fill up
+    the SIMD lanes times the vector units, but not more than
+    that to avoid stack overflow and cache trashing.
+  */
+  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1] =
+	  dev->max_work_item_sizes[2] = dev->max_work_group_size = 1024*4;
+
   dev->preferred_wg_size_multiple = 8;
   dev->preferred_vector_width_char = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_CHAR;
   dev->preferred_vector_width_short = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_SHORT;
@@ -239,20 +252,22 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->max_clock_frequency = 0;
   dev->address_bits = POCL_DEVICE_ADDRESS_BITS;
 
+  dev->image_support = CL_TRUE;
   /* Use the minimum values until we get a more sensible
      upper limit from somewhere. */
   dev->max_read_image_args = dev->max_write_image_args = 128;
   dev->image2d_max_width = dev->image2d_max_height = 8192;
   dev->image3d_max_width = dev->image3d_max_height = dev->image3d_max_depth = 2048;
+  dev->image_max_buffer_size = 65536;
+  dev->image_max_array_size = 2048;
   dev->max_samplers = 16;
   dev->max_constant_args = 8;
 
   dev->max_mem_alloc_size = 0;
-  dev->image_support = CL_TRUE;
-  dev->image_max_buffer_size = 0;
-  dev->image_max_array_size = 0;
+
   dev->max_parameter_size = 1024;
-  dev->min_data_type_align_size = dev->mem_base_addr_align = MAX_EXTENDED_ALIGNMENT;
+  dev->min_data_type_align_size = MAX_EXTENDED_ALIGNMENT; // this is in bytes
+  dev->mem_base_addr_align = MAX_EXTENDED_ALIGNMENT*8; // this is in bits
   dev->half_fp_config = 0;
   dev->single_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
   dev->double_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
@@ -265,15 +280,29 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->local_mem_size = 0;
   dev->error_correction_support = CL_FALSE;
   dev->host_unified_memory = CL_TRUE;
-  dev->profiling_timer_resolution = 0;
+
+  dev->profiling_timer_resolution = pocl_timer_resolution;
+
   dev->endian_little = !(WORDS_BIGENDIAN);
   dev->available = CL_TRUE;
   dev->compiler_available = CL_TRUE;
+  dev->spmd = CL_FALSE;
   dev->execution_capabilities = CL_EXEC_KERNEL | CL_EXEC_NATIVE_KERNEL;
-  dev->queue_properties = CL_QUEUE_PROFILING_ENABLE;
   dev->platform = 0;
-  dev->device_partition_properties[0] = 0;
-  dev->printf_buffer_size = 0;
+
+  dev->parent_device = NULL;
+  // basic does not support partitioning
+  dev->max_sub_devices = 1;
+  dev->num_partition_properties = 1;
+  dev->partition_properties = calloc(dev->num_partition_properties,
+    sizeof(cl_device_partition_property));
+  dev->num_partition_types = 0;
+  dev->partition_type = NULL;
+
+  /* printf buffer size is meaningless for pocl, so just set it to
+   * the minimum value required by the spec
+   */
+  dev->printf_buffer_size = 1024*1024 ;
   dev->vendor = "pocl";
   dev->profile = "FULL_PROFILE";
   /* Note: The specification describes identifiers being delimited by
@@ -282,23 +311,34 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
      ensure that there is no more than a single space between
      identifiers. */
 
-#ifndef _CL_DISABLE_LONG
-#define DOUBLE_EXT "cl_khr_fp64 "
-#else
-#define DOUBLE_EXT 
-#endif
+  dev->should_allocate_svm = 0;
+  /* OpenCL 2.0 properties */
+  dev->svm_caps = CL_DEVICE_SVM_COARSE_GRAIN_BUFFER
+                  | CL_DEVICE_SVM_FINE_GRAIN_BUFFER
+                  | CL_DEVICE_SVM_ATOMICS;
+  /* TODO these are minimums, figure out whats a reasonable value */
+  dev->max_events = 1024;
+  dev->max_queues = 1;
+  dev->max_pipe_args = 16;
+  dev->max_pipe_active_res = 1;
+  dev->max_pipe_packet_size = 1024;
+  dev->dev_queue_pref_size = 16 * 1024;
+  dev->dev_queue_max_size = 256 * 1024;
+  dev->on_dev_queue_props = CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE
+                               | CL_QUEUE_PROFILING_ENABLE;
+  dev->on_host_queue_props = CL_QUEUE_PROFILING_ENABLE;
 
-#ifndef _CL_DISABLE_HALF
-#define HALF_EXT "cl_khr_fp16 "
-#else
-#define HALF_EXT
-#endif
 
-  dev->extensions = DOUBLE_EXT HALF_EXT "cl_khr_byte_addressable_store";
+  dev->extensions = HOST_DEVICE_EXTENSIONS;
 
   dev->llvm_target_triplet = OCL_KERNEL_TARGET;
+#ifdef POCL_BUILT_WITH_CMAKE
+  dev->llvm_cpu = get_cpu_name();
+#else
   dev->llvm_cpu = OCL_KERNEL_TARGET_CPU;
+#endif
   dev->has_64bit_long = 1;
+  dev->autolocals_to_args = 1;
 }
 
 unsigned int
@@ -313,12 +353,15 @@ pocl_basic_probe(struct pocl_device_ops *ops)
   return env_count;
 }
 
+
+
 void
 pocl_basic_init (cl_device_id device, const char* parameters)
 {
   struct data *d;
   static int global_mem_id;
   static int first_basic_init = 1;
+  static int device_number = 0;
   
   if (first_basic_init)
     {
@@ -332,8 +375,25 @@ pocl_basic_init (cl_device_id device, const char* parameters)
   d->current_kernel = NULL;
   d->current_dlhandle = 0;
   device->data = d;
+  /* hwloc probes OpenCL device info at its initialization in case
+     the OpenCL extension is enabled. This causes to printout 
+     an unimplemented property error because hwloc is used to
+     initialize global_mem_size which it is not yet. Just put 
+     a nonzero there for now. */
+  device->global_mem_size = 1;
   pocl_topology_detect_device_info(device);
   pocl_cpuinfo_detect_device_info(device);
+  pocl_set_buffer_image_limits(device);
+
+  /* in case hwloc doesn't provide a PCI ID, let's generate
+     a vendor id that hopefully is unique across vendors. */
+  const char *magic = "pocl";
+  if (device->vendor_id == 0)
+    device->vendor_id =
+      magic[0] | magic[1] << 8 | magic[2] << 16 | magic[3] << 24;
+
+  device->vendor_id += device_number;
+  device_number++;
 
   /* The basic driver represents only one "compute unit" as
      it doesn't exploit multiple hardware threads. Multiple
@@ -350,55 +410,35 @@ pocl_basic_init (cl_device_id device, const char* parameters)
   #endif
 }
 
-void *
-pocl_basic_malloc (void *device_data, cl_mem_flags flags,
-		    size_t size, void *host_ptr)
-{
-  void *b;
-
-  if (flags & CL_MEM_COPY_HOST_PTR)
-    {
-      if (posix_memalign (&b, MAX_EXTENDED_ALIGNMENT, size) == 0)
-        {
-          memcpy (b, host_ptr, size);
-          return b;
-        }
-      
-      return NULL;
-    }
-  
-  if (flags & CL_MEM_USE_HOST_PTR && host_ptr != NULL)
-    {
-      return host_ptr;
-    }
-
-  if (posix_memalign (&b, MAX_EXTENDED_ALIGNMENT, size) == 0)
-    return b;
-  
-  return NULL;
-}
-
 cl_int
 pocl_basic_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
 {
   void *b = NULL;
-  struct data* d = (struct data*)device->data;
-  cl_int flags = mem_obj->flags;
+  cl_mem_flags flags = mem_obj->flags;
 
   /* if memory for this global memory is not yet allocated -> do it */
   if (mem_obj->device_ptrs[device->global_mem_id].mem_ptr == NULL)
     {
-      if (flags & CL_MEM_USE_HOST_PTR && mem_obj->mem_host_ptr != NULL)
+      if (flags & CL_MEM_USE_HOST_PTR)
         {
+          // mem_host_ptr must be non-NULL
+          assert(mem_obj->mem_host_ptr != NULL);
           b = mem_obj->mem_host_ptr;
         }
-      else if (posix_memalign (&b, MAX_EXTENDED_ALIGNMENT, 
-                               mem_obj->size) != 0)
-        return CL_MEM_OBJECT_ALLOCATION_FAILURE;
+      else
+        {
+          b = pocl_memalign_alloc_global_mem(device, MAX_EXTENDED_ALIGNMENT, mem_obj->size);
+          if (b == NULL)
+            return CL_MEM_OBJECT_ALLOCATION_FAILURE;
+        }
 
       if (flags & CL_MEM_COPY_HOST_PTR)
-        memcpy (b, mem_obj->mem_host_ptr, mem_obj->size);
-    
+        {
+          // mem_host_ptr must be non-NULL
+          assert(mem_obj->mem_host_ptr != NULL);
+          memcpy (b, mem_obj->mem_host_ptr, mem_obj->size);
+        }
+
       mem_obj->device_ptrs[device->global_mem_id].mem_ptr = b;
       mem_obj->device_ptrs[device->global_mem_id].global_mem_id = 
         device->global_mem_id;
@@ -406,35 +446,49 @@ pocl_basic_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
   /* copy already allocated global mem info to devices own slot */
   mem_obj->device_ptrs[device->dev_id] = 
     mem_obj->device_ptrs[device->global_mem_id];
-    
+
   return CL_SUCCESS;
 }
 
 void
-pocl_basic_free (void *data, cl_mem_flags flags, void *ptr)
+pocl_basic_free (cl_device_id device, cl_mem memobj)
 {
+  cl_mem_flags flags = memobj->flags;
+
   if (flags & CL_MEM_USE_HOST_PTR)
     return;
-  
-  free (ptr);
+
+  void* ptr = memobj->device_ptrs[device->dev_id].mem_ptr;
+  size_t size = memobj->size;
+
+  pocl_free_global_mem(device, ptr, size);
+}
+
+void pocl_basic_free_ptr (cl_device_id device, void* mem_ptr)
+{
+  /* TODO we should somehow figure out the size argument
+   * and call pocl_free_global_mem */
+  POCL_MEM_FREE(mem_ptr);
 }
 
 void
-pocl_basic_read (void *data, void *host_ptr, const void *device_ptr, size_t cb)
+pocl_basic_read (void *data, void *host_ptr, const void *device_ptr, 
+                 size_t offset, size_t cb)
 {
   if (host_ptr == device_ptr)
     return;
 
-  memcpy (host_ptr, device_ptr, cb);
+  memcpy (host_ptr, (char*)device_ptr + offset, cb);
 }
 
 void
-pocl_basic_write (void *data, const void *host_ptr, void *device_ptr, size_t cb)
+pocl_basic_write (void *data, const void *host_ptr, void *device_ptr, 
+                  size_t offset, size_t cb)
 {
   if (host_ptr == device_ptr)
     return;
 
-  memcpy (device_ptr, host_ptr, cb);
+  memcpy ((char*)device_ptr + offset, host_ptr, cb);
 }
 
 
@@ -444,9 +498,6 @@ pocl_basic_run
  _cl_command_node* cmd)
 {
   struct data *d;
-  const char *module_fn;
-  char workgroup_string[WORKGROUP_STRING_LENGTH];
-  unsigned device;
   struct pocl_argument *al;
   size_t x, y, z;
   unsigned i;
@@ -458,18 +509,9 @@ pocl_basic_run
 
   d->current_kernel = kernel;
 
-  /* Find which device number within the context correspond
-     to current device.  */
-  for (i = 0; i < kernel->context->num_devices; ++i)
-    {
-      if (kernel->context->devices[i]->data == data)
-        {
-          device = i;
-          break;
-        }
-    }
-
-  void *arguments[kernel->num_args + kernel->num_locals];
+  void **arguments = (void**)malloc(
+      sizeof(void*) * (kernel->num_args + kernel->num_locals)
+    );
 
   /* Process the kernel arguments. Convert the opaque buffer
      pointers to real device pointers, allocate dynamic local 
@@ -480,7 +522,7 @@ pocl_basic_run
       if (kernel->arg_info[i].is_local)
         {
           arguments[i] = malloc (sizeof (void *));
-          *(void **)(arguments[i]) = pocl_basic_malloc(data, 0, al->size, NULL);
+          *(void **)(arguments[i]) = pocl_memalign_alloc(MAX_EXTENDED_ALIGNMENT, al->size);
         }
       else if (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER)
         {
@@ -494,26 +536,27 @@ pocl_basic_run
               *(void **)arguments[i] = NULL;
             }
           else
-            arguments[i] = &((*(cl_mem *) (al->value))->device_ptrs[device].mem_ptr);
+            arguments[i] = &((*(cl_mem *) (al->value))->device_ptrs[cmd->device->dev_id].mem_ptr);
         }
       else if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE)
         {
           dev_image_t di;
-          fill_dev_image_t (&di, al, device);
+          fill_dev_image_t (&di, al, cmd->device);
 
-          void* devptr = pocl_basic_malloc (data, 0, sizeof(dev_image_t), NULL);
+          void* devptr = pocl_memalign_alloc(MAX_EXTENDED_ALIGNMENT,  sizeof(dev_image_t));
           arguments[i] = malloc (sizeof (void *));
           *(void **)(arguments[i]) = devptr; 
-          pocl_basic_write (data, &di, devptr, sizeof(dev_image_t));
+          pocl_basic_write (data, &di, devptr, 0, sizeof(dev_image_t));
         }
       else if (kernel->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
         {
           dev_sampler_t ds;
+          fill_dev_sampler_t(&ds, al);
           
+          void* devptr = pocl_memalign_alloc(MAX_EXTENDED_ALIGNMENT, sizeof(dev_sampler_t));
           arguments[i] = malloc (sizeof (void *));
-          *(void **)(arguments[i]) = pocl_basic_malloc 
-            (data, 0, sizeof(dev_sampler_t), NULL);
-          pocl_basic_write (data, &ds, *(void**)arguments[i], sizeof(dev_sampler_t));
+          *(void **)(arguments[i]) = devptr;
+          pocl_basic_write (data, &ds, devptr, 0, sizeof(dev_sampler_t));
         }
       else
         {
@@ -526,7 +569,7 @@ pocl_basic_run
     {
       al = &(cmd->command.run.arguments[i]);
       arguments[i] = malloc (sizeof (void *));
-      *(void **)(arguments[i]) = pocl_basic_malloc (data, 0, al->size, NULL);
+      *(void **)(arguments[i]) = pocl_memalign_alloc(MAX_EXTENDED_ALIGNMENT, al->size);
     }
 
   for (z = 0; z < pc->num_groups[2]; ++z)
@@ -548,27 +591,28 @@ pocl_basic_run
     {
       if (kernel->arg_info[i].is_local)
         {
-          pocl_basic_free (data, 0, *(void **)(arguments[i]));
-          free (arguments[i]);
+          POCL_MEM_FREE(*(void **)(arguments[i]));
+          POCL_MEM_FREE(arguments[i]);
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE)
+      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE ||
+                kernel->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
         {
-          pocl_basic_free (data, 0, *(void **)(arguments[i]));
-          free (arguments[i]);            
+          POCL_MEM_FREE(*(void **)(arguments[i]));
+          POCL_MEM_FREE(arguments[i]);
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_SAMPLER || 
-               (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER && *(void**)arguments[i] == NULL))
+      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER && *(void**)arguments[i] == NULL)
         {
-          free (arguments[i]);
+          POCL_MEM_FREE(arguments[i]);
         }
     }
   for (i = kernel->num_args;
        i < kernel->num_args + kernel->num_locals;
        ++i)
     {
-      pocl_basic_free(data, 0, *(void **)(arguments[i]));
-      free (arguments[i]);
+      POCL_MEM_FREE(*(void **)(arguments[i]));
+      POCL_MEM_FREE(arguments[i]);
     }
+  free(arguments);
 }
 
 void
@@ -580,12 +624,13 @@ pocl_basic_run_native
 }
 
 void
-pocl_basic_copy (void *data, const void *src_ptr, void *__restrict__ dst_ptr, size_t cb)
+pocl_basic_copy (void *data, const void *src_ptr, size_t src_offset, 
+                 void *__restrict__ dst_ptr, size_t dst_offset, size_t cb)
 {
   if (src_ptr == dst_ptr)
     return;
   
-  memcpy (dst_ptr, src_ptr, cb);
+  memcpy ((char*)dst_ptr + dst_offset, (char*)src_ptr + src_offset, cb);
 }
 
 void
@@ -662,10 +707,10 @@ pocl_basic_read_rect (void *data,
 {
   char const *__restrict const adjusted_device_ptr = 
     (char const*)device_ptr +
-    buffer_origin[0] + buffer_row_pitch * (buffer_origin[1] + buffer_slice_pitch * buffer_origin[2]);
+    buffer_origin[2] * buffer_slice_pitch + buffer_origin[1] * buffer_row_pitch + buffer_origin[0];
   char *__restrict__ const adjusted_host_ptr = 
     (char*)host_ptr +
-    host_origin[0] + host_row_pitch * (host_origin[1] + host_slice_pitch * host_origin[2]);
+    host_origin[2] * host_slice_pitch + host_origin[1] * host_row_pitch + host_origin[0];
   
   size_t j, k;
   
@@ -705,6 +750,83 @@ pocl_basic_fill_rect (void *data,
                 + buffer_slice_pitch * k, fill_pixel, pixel_size);
 }
 
+void pocl_basic_memfill(void *ptr,
+                        size_t size,
+                        size_t offset,
+                        const void* pattern,
+                        size_t pattern_size)
+{
+  size_t i;
+  unsigned j;
+
+  switch (pattern_size)
+    {
+    case 1:
+      {
+      uint8_t * p = (uint8_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        p[i] = *(uint8_t*)pattern;
+      }
+      break;
+    case 2:
+      {
+      uint16_t * p = (uint16_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        p[i] = *(uint16_t*)pattern;
+      }
+      break;
+    case 4:
+      {
+      uint32_t * p = (uint32_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        p[i] = *(uint32_t*)pattern;
+      }
+      break;
+    case 8:
+      {
+      uint64_t * p = (uint64_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        p[i] = *(uint64_t*)pattern;
+      }
+      break;
+    case 16:
+      {
+      uint64_t * p = (uint64_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        for (j = 0; j < 2; j++)
+          p[(i<<1) + j] = *((uint64_t*)pattern + j);
+      }
+      break;
+    case 32:
+      {
+      uint64_t * p = (uint64_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        for (j = 0; j < 4; j++)
+          p[(i<<2) + j] = *((uint64_t*)pattern + j);
+      }
+      break;
+    case 64:
+      {
+      uint64_t * p = (uint64_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        for (j = 0; j < 8; j++)
+          p[(i<<3) + j] = *((uint64_t*)pattern + j);
+      }
+      break;
+    case 128:
+      {
+      uint64_t * p = (uint64_t*)ptr + offset;
+      for (i = 0; i < size; i++)
+        for (j = 0; j < 16; j++)
+          p[(i<<4) + j] = *((uint64_t*)pattern + j);
+      }
+      break;
+    default:
+      assert (0 && "Invalid pattern size");
+      break;
+    }
+}
+
 void *
 pocl_basic_map_mem (void *data, void *buf_ptr, 
                       size_t offset, size_t size,
@@ -713,29 +835,35 @@ pocl_basic_map_mem (void *data, void *buf_ptr,
   /* All global pointers of the pthread/CPU device are in 
      the host address space already, and up to date. */
   if (host_ptr != NULL) return host_ptr;
-  return buf_ptr + offset;
+  return (char*)buf_ptr + offset;
 }
 
+void* pocl_basic_unmap_mem(void *data, void *host_ptr,
+                           void *device_start_ptr,
+                           size_t size)
+{
+  return host_ptr;
+}
+
+
 void
 pocl_basic_uninit (cl_device_id device)
 {
   struct data *d = (struct data*)device->data;
-  free (d);
+  POCL_MEM_FREE(d);
   device->data = NULL;
 }
 
 cl_ulong
 pocl_basic_get_timer_value (void *data) 
 {
-  struct timeval current;
-  gettimeofday(&current, NULL);  
-  return (current.tv_sec * 1000000 + current.tv_usec)*1000;
+  return pocl_gettimemono_ns();
 }
 
 cl_int 
 pocl_basic_get_supported_image_formats (cl_mem_flags flags,
                                         const cl_image_format **image_formats,
-                                        cl_int *num_img_formats)
+                                        cl_uint *num_img_formats)
 {
     if (num_img_formats == NULL || image_formats == NULL)
       return CL_INVALID_VALUE;
@@ -772,21 +900,22 @@ void check_compiler_cache (_cl_command_node *cmd)
     {
       if (strcmp (ci->tmp_dir, cmd->command.run.tmp_dir) == 0 &&
           strcmp (ci->function_name, 
-                  cmd->command.run.kernel->function_name) == 0)
+                  cmd->command.run.kernel->name) == 0)
         {
           POCL_UNLOCK (compiler_cache_lock);
           cmd->command.run.wg = ci->wg;
           return;
         }
     }
-  ci = malloc (sizeof (compiler_cache_item));
+
+  ci = (compiler_cache_item*) malloc (sizeof (compiler_cache_item));
   ci->next = NULL;
   ci->tmp_dir = strdup(cmd->command.run.tmp_dir);
-  ci->function_name = strdup (cmd->command.run.kernel->function_name);
+  ci->function_name = strdup (cmd->command.run.kernel->name);
   const char* module_fn = llvm_codegen (cmd->command.run.tmp_dir,
                                         cmd->command.run.kernel,
                                         cmd->device);
-  dlhandle = lt_dlopen (module_fn);     
+  dlhandle = lt_dlopen (module_fn);
   if (dlhandle == NULL)
     {
       printf ("pocl error: lt_dlopen(\"%s\") failed with '%s'.\n", 
@@ -796,7 +925,7 @@ void check_compiler_cache (_cl_command_node *cmd)
       abort();
     }
   snprintf (workgroup_string, WORKGROUP_STRING_LENGTH,
-            "_%s_workgroup", cmd->command.run.kernel->function_name);
+            "_pocl_launcher_%s_workgroup", cmd->command.run.kernel->name);
   cmd->command.run.wg = ci->wg = 
     (pocl_workgroup) lt_dlsym (dlhandle, workgroup_string);
 
