@@ -24,7 +24,12 @@
 
 #include <pocl_cl.h>
 #include <string.h>
-#include <unistd.h>
+
+#ifndef _MSC_VER
+#  include <unistd.h>
+#else
+#  include "vccompat.hpp"
+#endif
 
 #include "config.h"
 #include "cpuinfo.h"
@@ -40,14 +45,30 @@ const char* cpufreq_file="/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq"
 #if   defined  __powerpc__
  #define FREQSTRING "clock"
  #define MODELSTRING "cpu\t"
+ #define DEFAULTVENDOR "AIM" // Apple-IBM-Motorola
+ #define DEFAULTVENDORID 0x1014 // IBM
+ #define VENDORSTRING "vendor"
 #elif defined __arm__
  #define FREQSTRING " "
  #define MODELSTRING "Processor"
+ #define DEFAULTVENDOR "ARM"
+ #define DEFAULTVENDORID 0x13b5 // ARM
+ #define VENDORSTRING "CPU implementer"
+#elif defined __mips
+ #define FREQSTRING "BogoMIPS\t"
+ #define MODELSTRING "cpu model\t"
+ #define DEFAULTVENDORID 0x153f // MIPS
+ #define DEFAULTVENDOR "MIPS"
 #else
  #define FREQSTRING "cpu MHz"
  #define MODELSTRING "model name"
+ #define DEFAULTVENDOR "Unknown x86"
+ #define DEFAULTVENDORID 0x0
+ #define VENDORSTRING "vendor_id"
 #endif
 
+static const char *cpuvendor_default = DEFAULTVENDOR;
+
 /** 
  * Read the CPU maximum frequency from the Linux cpufreq interface.
  * If cpufreq is not available on current host processor, returns -1.
@@ -84,7 +105,8 @@ int pocl_cpufreq_get_max()
  * @return The clock frequency in MHz, or -1 if couldn't figure it out.
  */
 int
-pocl_cpuinfo_detect_max_clock_frequency() {
+pocl_cpuinfo_detect_max_clock_frequency()
+{
   int cpufreq=-1;
   
   // First try to get the result from cpufreq interface.
@@ -139,8 +161,8 @@ pocl_cpuinfo_detect_max_clock_frequency() {
  * @return The number of hardware threads, or -1 if couldn't figure it out.
  */
 int
-pocl_cpuinfo_detect_compute_unit_count() {
-
+pocl_cpuinfo_detect_compute_unit_count()
+{
   if (access (cpuinfo, R_OK) != 0) 
       return -1;
   else 
@@ -157,7 +179,7 @@ pocl_cpuinfo_detect_compute_unit_count() {
          system. In Meego Harmattan on ARM it prints Processor instead of
          processor */
       char* p = contents;
-      while ((p = strstr (p, "rocessor")) != NULL) 
+      while ((p = strstr (p, "rocessor")) != NULL)
         {
           cores++;
           /* Skip to the end of the line. Otherwise causes two cores
@@ -210,30 +232,86 @@ pocl_cpuinfo_detect_compute_unit_count() {
   return -1;  
 }
 
+#ifdef POCL_ANDROID
+
+#define SYSFS_CPU_NUM_CORES_NODE    "/sys/devices/system/cpu/possible"
+
+int
+pocl_sysfs_detect_compute_unit_count()
+{
+  int cores = -1;
+
+  FILE *fp = fopen(SYSFS_CPU_NUM_CORES_NODE, "r");
+
+  // cpu/possible will of format
+  // 0        : for single-core devices
+  // 0-(n-1)  : for n-core cpus
+  if (fp)
+    {
+      cores = fgetc(fp) - '0';
+      if (!feof(fp))         // If more than 1 cores
+        {
+          fgetc(fp);          // Ignore '-'
+          fscanf(fp, "%d", &cores);
+        }
+      fclose(fp);
+      cores ++;             // always printed as (n-1)
+    }
+
+  return cores;
+}
+#endif
+
 void
-pocl_cpuinfo_append_cpu_name(cl_device_id device)
+pocl_cpuinfo_get_cpu_name_and_vendor(cl_device_id device)
 {
   /* If something fails here, have this as backup solution.
    * short_name is in the .data anyways.*/
   device->long_name = device->short_name;
-  
+
+  /* default vendor and vendor_id, in case it cannot be found by other means */
+  device->vendor = cpuvendor_default;
+  if (device->vendor_id == 0)
+    device->vendor_id = DEFAULTVENDORID;
+
   /* read contents of /proc/cpuinfo */
-  if (access (cpuinfo, R_OK) != 0) 
+  if (access (cpuinfo, R_OK) != 0)
     return;
   FILE *f = fopen (cpuinfo, "r");
   char contents[MAX_CPUINFO_SIZE];
   int num_read = fread (contents, 1, MAX_CPUINFO_SIZE - 1, f);            
+  fclose(f);
   contents[num_read]='\0';
 
+  char *start, *end;
+  /* find the vendor_id string an put */
+#ifdef VENDORSTRING
+  do {
+    start = strstr(contents, VENDORSTRING"\t: ");
+    if (!start)
+      break;
+    start += strlen(VENDORSTRING"\t: ");
+    end = strchr(start, '\n');
+    if (!end)
+      break;
+    char *_vendor = malloc(end-start + 1);
+    if (!_vendor)
+      break;
+    memcpy(_vendor, start, end-start);
+    _vendor[end-start] = '\0';
+    device->vendor = _vendor;
+  } while (0);
+#endif
+
   /* find the cpu description */
-  char *start=strstr (contents, MODELSTRING"\t: ");
-  if (start == NULL) 
+  start=strstr (contents, MODELSTRING"\t: ");
+  if (start == NULL)
     return;
   start += strlen (MODELSTRING"\t: ");
-  char *end = strchr (start, '\n'); 
-  if (end == NULL) 
+  end = strchr (start, '\n');
+  if (end == NULL)
     return;
-  
+
   /* create the descriptive long_name for device */
   int len = strlen (device->short_name) + (end-start) + 2;
   char *new_name = (char*)malloc (len);
@@ -245,13 +323,21 @@ pocl_cpuinfo_append_cpu_name(cl_device_id device)
 void
 pocl_cpuinfo_detect_device_info(cl_device_id device) 
 {
+  int res;
+#ifdef POCL_ANDROID
+  /* sysfs node seems more suitable for android kernels
+     override the value provided by hwloc
+   */
+  device->max_compute_units = pocl_sysfs_detect_compute_unit_count();
+#else
   if (device->max_compute_units == 0) {
-    if ((device->max_compute_units = pocl_cpuinfo_detect_compute_unit_count()) == -1)
-      device->max_compute_units = 0;
+    res = pocl_cpuinfo_detect_compute_unit_count();
+    device->max_compute_units = (res > 0) ? (cl_uint)res : 0;
   }
+#endif
 
-  if ((device->max_clock_frequency = pocl_cpuinfo_detect_max_clock_frequency()) == -1)
-    device->max_clock_frequency = 0;
+  res = pocl_cpuinfo_detect_max_clock_frequency();
+  device->max_clock_frequency = (res > 0) ? (cl_uint)res : 0;
 
-  pocl_cpuinfo_append_cpu_name(device);
+  pocl_cpuinfo_get_cpu_name_and_vendor(device);
 }
