@@ -1,7 +1,7 @@
 /* Definition of available OpenCL devices.
 
    Copyright (c) 2011 Universidad Rey Juan Carlos and
-                 2012 Pekka Jääskeläinen / Tampere University of Technology
+                 2012-2015 Pekka Jääskeläinen / Tampere University of Technology
    
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -22,29 +22,37 @@
    THE SOFTWARE.
 */
 
-#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 
+#ifndef _MSC_VER
+#  include <unistd.h>
+#else
+#  include "vccompat.hpp"
+#endif
+
 #include "devices.h"
 #include "common.h"
+#include "pocl_runtime_config.h"
 #include "basic/basic.h"
 #include "pthread/pocl-pthread.h"
-
-#if defined(BUILD_SPU)
-#include "cellspu/cellspu.h"
-#endif
+#include "pocl_debug.h"
+#include "pocl_cache.h"
+#include "pocl_queue_util.h"
 
 #if defined(TCE_AVAILABLE)
 #include "tce/ttasim/ttasim.h"
 #endif
 
+#include "hsa/pocl-hsa.h"
+
 #define MAX_DEV_NAME_LEN 64
 
 /* the enabled devices */
 static struct _cl_device_id* pocl_devices = NULL;
 unsigned int pocl_num_devices = 0;
 
+
 /* Init function prototype */
 typedef void (*init_device_ops)(struct pocl_device_ops*);
 
@@ -52,17 +60,17 @@ typedef void (*init_device_ops)(struct pocl_device_ops*);
 static init_device_ops pocl_devices_init_ops[] = {
   pocl_pthread_init_device_ops,
   pocl_basic_init_device_ops,
-#if defined(BUILD_SPU)
-  pocl_cellspu_init_device_ops,
-#endif
 #if defined(TCE_AVAILABLE)
   pocl_ttasim_init_device_ops,
 #endif
+#if defined(BUILD_HSA)
+  pocl_hsa_init_device_ops,
+#endif
 };
 
 #define POCL_NUM_DEVICE_TYPES (sizeof(pocl_devices_init_ops) / sizeof((pocl_devices_init_ops)[0]))
 
-static struct pocl_device_ops pocl_device_ops[POCL_NUM_DEVICE_TYPES] = {0};
+static struct pocl_device_ops pocl_device_ops[POCL_NUM_DEVICE_TYPES];
 
 /**
  * Get the number of specified devices from environnement
@@ -70,7 +78,7 @@ static struct pocl_device_ops pocl_device_ops[POCL_NUM_DEVICE_TYPES] = {0};
 int pocl_device_get_env_count(const char *dev_type)
 {
   const char *dev_env = getenv(POCL_DEVICES_ENV);
-  char *ptr, *saveptr, *tofree, *token;
+  char *ptr, *saveptr = NULL, *tofree, *token;
   unsigned int dev_count = 0;
   if (dev_env == NULL) 
     {
@@ -83,17 +91,14 @@ int pocl_device_get_env_count(const char *dev_type)
         dev_count++;
       ptr = NULL;
     }
-  free (tofree);
+  POCL_MEM_FREE(tofree);
 
   return dev_count;
 }
 
-static unsigned int __total_dev_count = 0;
-
 unsigned int
 pocl_get_devices(cl_device_type device_type, struct _cl_device_id **devices, unsigned int num_devices)
 {
-  struct _cl_device_id *device;
   unsigned int i, dev_added = 0;
 
   for (i = 0; i < pocl_num_devices; ++i)
@@ -139,7 +144,7 @@ pocl_device_common_init(struct _cl_device_id* dev)
   POCL_INIT_OBJECT(dev);
   dev->driver_version = PACKAGE_VERSION;
   if(dev->version == NULL)
-    dev->version = "OpenCL 1.2 pocl";
+    dev->version = "OpenCL 2.0 pocl";
 
   dev->short_name = strdup(dev->ops->device_name);
   if(dev->long_name == NULL)
@@ -156,17 +161,44 @@ str_toupper(char *out, const char *in)
   out[i] = '\0';
 }
 
+static inline void
+pocl_string_to_dirname(char *str)
+{
+  char *s_ptr;
+  if (!str) return;
+
+  // Replace special characters with '_'
+  for (s_ptr = str; (*s_ptr); s_ptr++)
+    {
+      if (!isalnum(*s_ptr))
+        *s_ptr = '_';
+    }
+}
+
 void 
 pocl_init_devices()
 {
   static unsigned int init_done = 0;
+  static unsigned int init_in_progress = 0;
   static pocl_lock_t pocl_init_lock = POCL_LOCK_INITIALIZER;
 
-  int i, j, dev_index;
+  unsigned i, j, dev_index;
   char env_name[1024];
   char dev_name[MAX_DEV_NAME_LEN] = {0};
   unsigned int device_count[POCL_NUM_DEVICE_TYPES];
 
+  /* This is a workaround to a nasty problem with libhwloc: When
+     initializing basic, it calls libhwloc to query device info.
+     In case libhwloc has the OpenCL plugin installed, it initializes
+     it and it leads to initializing pocl again which leads to an
+     infinite loop. */
+
+  if (init_in_progress)
+      return;
+  init_in_progress = 1;
+
+  if (init_done == 0)
+    POCL_INIT_LOCK(pocl_init_lock);
   POCL_LOCK(pocl_init_lock);
   if (init_done) 
     {
@@ -174,6 +206,17 @@ pocl_init_devices()
       return;
     }
 
+  /* Set a global debug flag, so we don't have to call pocl_get_bool_option
+   * everytime we use the debug macros */
+#ifdef POCL_DEBUG_MESSAGES
+  pocl_debug_messages = pocl_get_bool_option("POCL_DEBUG", 0);
+  stderr_is_a_tty = isatty(fileno(stderr));
+#endif
+
+  pocl_cache_init_topdir();
+
+  pocl_init_queue_list();
+
   /* Init operations */
   for (i = 0; i < POCL_NUM_DEVICE_TYPES; ++i)
     {
@@ -187,18 +230,19 @@ pocl_init_devices()
     }
 
   assert(pocl_num_devices > 0);
-  pocl_devices = calloc(pocl_num_devices, sizeof(struct _cl_device_id));
+  pocl_devices = (struct _cl_device_id*) calloc(pocl_num_devices, sizeof(struct _cl_device_id));
   if (pocl_devices == NULL)
     POCL_ABORT("Can not allocate memory for devices\n");
 
   dev_index = 0;
-  /* Init infos for each probbed devices */
+  /* Init infos for each probed devices */
   for (i = 0; i < POCL_NUM_DEVICE_TYPES; ++i)
     {
       assert(pocl_device_ops[i].init);
       for (j = 0; j < device_count[i]; ++j)
         {
           pocl_devices[dev_index].ops = &pocl_device_ops[i];
+          pocl_devices[dev_index].dev_id = dev_index;
           /* The default value for the global memory space identifier is
              the same as the device id. The device instance can then override 
              it to point to some other device's global memory id in case of
@@ -219,6 +263,9 @@ pocl_init_devices()
 
           if (dev_index == 0)
             pocl_devices[dev_index].type |= CL_DEVICE_TYPE_DEFAULT;
+
+          pocl_devices[dev_index].cache_dir_name = strdup(pocl_devices[dev_index].long_name);
+          pocl_string_to_dirname(pocl_devices[dev_index].cache_dir_name);
           
           ++dev_index;
         }
