@@ -1,6 +1,6 @@
 /* tce_common.cc - common functionality over the different TCE/TTA device drivers.
 
-   Copyright (c) 2012-2013 Pekka Jääskeläinen / Tampere University of Technology
+   Copyright (c) 2012-2014 Pekka Jääskeläinen / Tampere University of Technology
    
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -26,8 +26,13 @@
 #include "config.h"
 #include "install-paths.h"
 #include "pocl_runtime_config.h"
+#include "pocl_hash.h"
 
-#include <unistd.h>
+#ifndef _MSC_VER
+#  include <unistd.h>
+#else
+#  include "vccompat.hpp"
+#endif
 
 /* Supress some warnings because of including tce_config.h after pocl's config.h. */
 #undef PACKAGE
@@ -44,11 +49,14 @@
 #include <DataLabel.hh>
 #include <AddressSpace.hh>
 #include <GlobalScope.hh>
+#include <Environment.hh>
+
+#include "pocl_cache.h"
 
 using namespace TTAMachine;
 
 #define COMMAND_LENGTH 256
-#define WORKGROUP_STRING_LENGTH 128
+#define WORKGROUP_STRING_LENGTH 1024
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
 #define ALIGNMENT (max(ALIGNOF_FLOAT16, ALIGNOF_DOUBLE16))
@@ -72,7 +80,7 @@ TCEDevice::~TCEDevice() {
 
 bool
 TCEDevice::isMultiCoreMachine() const {
-#if defined(TCEMC_AVAILABLE) && TCEMC_AVAILABLE == 1
+#ifdef TCEMC_AVAILABLE
   assert (machine_ != NULL);
   return machine_->coreCount() > 1;
 #else
@@ -279,31 +287,6 @@ TCEDevice::updateCurrentKernel(const _cl_command_run* runCmd,
   curLocalZ = runCmd->local_z;
 }
 
-cl_int
-pocl_tce_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
-{
-  void *b = NULL;
-  TCEDevice *d = (TCEDevice*)device->data;
-  cl_int flags = mem_obj->flags;
-
-  /* if memory for this global memory is not yet allocated -> do it */
-  if (mem_obj->device_ptrs[device->global_mem_id].mem_ptr == NULL)
-    {
-      b = pocl_tce_malloc
-        (device->data, flags, mem_obj->size, mem_obj->mem_host_ptr);
-      if (b == NULL) return CL_MEM_OBJECT_ALLOCATION_FAILURE;
-      mem_obj->device_ptrs[device->global_mem_id].mem_ptr = b;
-      mem_obj->device_ptrs[device->global_mem_id].global_mem_id = 
-        device->global_mem_id;
-    }
-  /* copy already allocated global mem info to devices own slot */
-  mem_obj->device_ptrs[device->dev_id] = 
-    mem_obj->device_ptrs[device->global_mem_id];
-    
-  return CL_SUCCESS;
-
-}
-
 void *
 pocl_tce_malloc (void *device_data, cl_mem_flags flags,
                  size_t size, void *host_ptr)
@@ -329,10 +312,35 @@ pocl_tce_malloc (void *device_data, cl_mem_flags flags,
   return (void*) chunk;
 }
 
-void
-pocl_tce_write (void *data, const void *host_ptr, void *device_ptr, size_t cb)
+cl_int
+pocl_tce_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
 {
-  TCEDevice* d = (TCEDevice*)data;
+  void *b = NULL;
+  cl_int flags = mem_obj->flags;
+
+  /* if memory for this global memory is not yet allocated -> do it */
+  if (mem_obj->device_ptrs[device->global_mem_id].mem_ptr == NULL)
+    {
+      b = pocl_tce_malloc
+        (device->data, flags, mem_obj->size, mem_obj->mem_host_ptr);
+      if (b == NULL) return CL_MEM_OBJECT_ALLOCATION_FAILURE;
+      mem_obj->device_ptrs[device->global_mem_id].mem_ptr = b;
+      mem_obj->device_ptrs[device->global_mem_id].global_mem_id = 
+        device->global_mem_id;
+    }
+  /* copy already allocated global mem info to devices own slot */
+  mem_obj->device_ptrs[device->dev_id] = 
+    mem_obj->device_ptrs[device->global_mem_id];
+    
+  return CL_SUCCESS;
+
+}
+
+void
+pocl_tce_write (void *data, const void *host_ptr, void *device_ptr, 
+                size_t /*offset*/, size_t cb)
+{
+  TCEDevice *d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
 #ifdef DEBUG_TTA_DRIVER
   printf("host: write %x %x %u\n", host_ptr, chunk->start_address, cb);
@@ -341,7 +349,8 @@ pocl_tce_write (void *data, const void *host_ptr, void *device_ptr, size_t cb)
 }
 
 void
-pocl_tce_read (void *data, void *host_ptr, const void *device_ptr, size_t cb)
+pocl_tce_read (void *data, void *host_ptr, const void *device_ptr, 
+               size_t /*offset*/, size_t cb)
 {
   TCEDevice* d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
@@ -370,8 +379,9 @@ pocl_tce_malloc_local (void *device_data, size_t size)
 }
 
 void
-pocl_tce_free (void */*data*/, cl_mem_flags /*flags*/, void *ptr)
+pocl_tce_free (cl_device_id device, cl_mem mem_obj)
 {
+  void* ptr = mem_obj->device_ptrs[device->dev_id].mem_ptr;
   free_chunk ((chunk_info_t*) ptr);
 }
 
@@ -383,7 +393,6 @@ pocl_tce_run
   TCEDevice *d = (TCEDevice*)data;
   int error;
   char bytecode[POCL_FILENAME_LENGTH];
-  char command[COMMAND_LENGTH];
   uint32_t kernelAddr;
   unsigned i;
 
@@ -404,7 +413,7 @@ pocl_tce_run
     if (access (assemblyFileName.c_str(), F_OK) != 0)
       {
         error = snprintf (bytecode, POCL_FILENAME_LENGTH,
-                          "%s/%s", cmd->command.run.tmp_dir, POCL_PARALLEL_BC_FILENAME);
+                          "%s%s", cmd->command.run.tmp_dir, POCL_PARALLEL_BC_FILENAME);
         TCEString buildCmd = 
           d->tceccCommandLine(&cmd->command.run, bytecode, assemblyFileName);
         
@@ -594,34 +603,43 @@ pocl_tce_map_mem (void *data, void *buf_ptr,
     } 
   else
     {
-      posix_memalign (&target, ALIGNMENT, size);
+        if (posix_memalign (&target, ALIGNMENT, size) != 0)
+        {
+            POCL_ABORT ("Could not allocate memory.");
+        }
     }
 
   /* Synch the device global region to the host memory. */
-  pocl_tce_read (data, target, chunk, size);
+  pocl_tce_read (data, target, chunk, 0, size);
   return target;
 }
 
 char* 
-pocl_tce_init_build(void *data, const char *dev_tmpdir) 
+pocl_tce_init_build(void *data)
 {
   TCEDevice *tce_dev = (TCEDevice*)data;
+  TCEString mach_tmpdir =
+      Environment::llvmtceCachePath();
+
+  TCEString mach_header_base =
+      mach_tmpdir + "/" + tce_dev->machine_->hash();
 
   int error = 0;
+
+  std::string devextHeaderFn =
+    std::string(mach_header_base) + std::string("_opencl_devext.h");
+
   /* Generate the vendor extensions header to provide explicit
      access to the (custom) hardware operations. */
-  std::string tceopgenCmd = 
-    std::string("tceopgen > ") + std::string(dev_tmpdir) + "/tceops.h";
-  
+  std::string tceopgenCmd =
+      std::string("tceopgen > ") + devextHeaderFn;
+
   error = system (tceopgenCmd.c_str());
   if (error == -1) return NULL;
 
-  std::string devextHeaderFn =
-    std::string(dev_tmpdir) + std::string("/_devext.h");
-
   std::string extgenCmd = 
     std::string("tceoclextgen ") + tce_dev->machine_file + 
-    std::string(" > ") + devextHeaderFn;
+      std::string(" >> ") + devextHeaderFn;
 
   error = system (extgenCmd.c_str());
   if (error == -1) return NULL;
@@ -637,13 +655,42 @@ pocl_tce_init_build(void *data, const char *dev_tmpdir)
   return include_switch;
 }
 
-void
-pocl_tce_copy (void */*data*/, const void *src_ptr, void *__restrict__ dst_ptr, size_t cb)
+void 
+pocl_tce_build_hash (void *data, SHA1_CTX *build_hash)
 {
-  POCL_ABORT_UNIMPLEMENTED();
+  TCEDevice *tce_dev = (TCEDevice*)data;
+  FILE* adf_file = fopen (tce_dev->machine_file.c_str(), "r");
+  size_t size;
+  uint8_t* adf_data = 0;
+  const char *extra_flags = NULL;
+  size_t ef_size;
+
+  fseek (adf_file, 0 , SEEK_END);
+  size = ftell (adf_file);
+  fseek (adf_file, 0, SEEK_SET);
+  adf_data = (uint8_t*)malloc (size);
+  if (fread (adf_data, 1, size, adf_file) == 0)
+      POCL_ABORT ("Could not read ADF.");
+  
+  //TCEString machine_hash = tce->dev->hash();
+  pocl_SHA1_Update (build_hash, adf_data, size);
+
+  if (pocl_is_option_set("POCL_TCECC_EXTRA_FLAGS"))
+    {
+      extra_flags = pocl_get_string_option("POCL_TCECC_EXTRA_FLAGS", "");
+      ef_size = strlen (extra_flags);
+      pocl_SHA1_Update (build_hash, (uint8_t*)extra_flags, ef_size);
+    }
+}
+
+void
+pocl_tce_copy (void */*data*/, const void *src_ptr, size_t /*src_offset*/,
+               void *__restrict__ dst_ptr, size_t /*dst_offset*/, size_t cb)
+{
+  POCL_ABORT_UNIMPLEMENTED("Copy not yet supported in TCE driver.");
   if (src_ptr == dst_ptr)
     return;
-  
+
   memcpy (dst_ptr, src_ptr, cb);
 }
 
@@ -668,7 +715,7 @@ pocl_tce_copy_rect (void */*data*/,
   
   size_t j, k;
 
-  POCL_ABORT_UNIMPLEMENTED();
+  POCL_ABORT_UNIMPLEMENTED("Copy rect not yet supported in TCE driver.");
 
   /* TODO: handle overlaping regions */
   
@@ -680,7 +727,7 @@ pocl_tce_copy_rect (void */*data*/,
 }
 
 void
-pocl_tce_write_rect (void */*data*/,
+pocl_tce_write_rect (void *data,
                      const void *__restrict__ const host_ptr,
                      void *__restrict__ const device_ptr,
                      const size_t *__restrict__ const buffer_origin,
@@ -691,27 +738,30 @@ pocl_tce_write_rect (void */*data*/,
                      size_t const host_row_pitch,
                      size_t const host_slice_pitch)
 {
-  char *__restrict const adjusted_device_ptr = 
-    (char*)device_ptr +
-    buffer_origin[0] + buffer_row_pitch * (buffer_origin[1] + buffer_slice_pitch * buffer_origin[2]);
   char const *__restrict__ const adjusted_host_ptr = 
     (char const*)host_ptr +
-    host_origin[0] + host_row_pitch * (host_origin[1] + host_slice_pitch * host_origin[2]);
+    host_origin[0] + host_row_pitch * host_origin[1] + 
+    host_slice_pitch * host_origin[2];
   
   size_t j, k;
-
+  size_t base_offset = buffer_origin[0] + buffer_row_pitch * buffer_origin[1] 
+    + buffer_slice_pitch * buffer_origin[2];
   /* TODO: handle overlaping regions */
-  POCL_ABORT_UNIMPLEMENTED();
-  
+    
   for (k = 0; k < region[2]; ++k)
     for (j = 0; j < region[1]; ++j)
-      memcpy (adjusted_device_ptr + buffer_row_pitch * j + buffer_slice_pitch * k,
-              adjusted_host_ptr + host_row_pitch * j + host_slice_pitch * k,
-              region[0]);
+      {
+        char const *__restrict h_ptr = adjusted_host_ptr + host_row_pitch * j 
+          + host_slice_pitch * k;       
+        size_t offset = base_offset + buffer_row_pitch * j 
+          + buffer_slice_pitch * k;
+        pocl_tce_write (data, h_ptr, device_ptr, offset, region[0]);
+  
+      }
 }
 
 void
-pocl_tce_read_rect (void */*data*/,
+pocl_tce_read_rect (void *data,
                     void *__restrict__ const host_ptr,
                     void *__restrict__ const device_ptr,
                     const size_t *__restrict__ const buffer_origin,
@@ -722,21 +772,23 @@ pocl_tce_read_rect (void */*data*/,
                     size_t const host_row_pitch,
                     size_t const host_slice_pitch)
 {
-  char const *__restrict const adjusted_device_ptr = 
-    (char const*)device_ptr +
-    buffer_origin[0] + buffer_row_pitch * (buffer_origin[1] + buffer_slice_pitch * buffer_origin[2]);
   char *__restrict__ const adjusted_host_ptr = 
     (char*)host_ptr +
-    host_origin[0] + host_row_pitch * (host_origin[1] + host_slice_pitch * host_origin[2]);
+    host_origin[0] + host_row_pitch * host_origin[1] + 
+    host_slice_pitch * host_origin[2];
   
   size_t j, k;
-  
-  /* TODO: handle overlaping regions */
-  POCL_ABORT_UNIMPLEMENTED();
+  size_t base_offset = buffer_origin[0] + buffer_row_pitch * buffer_origin[1] 
+    + buffer_slice_pitch * buffer_origin[2];
+  /* TODO: handle overlaping regions */ 
   
   for (k = 0; k < region[2]; ++k)
     for (j = 0; j < region[1]; ++j)
-      memcpy (adjusted_host_ptr + host_row_pitch * j + host_slice_pitch * k,
-              adjusted_device_ptr + buffer_row_pitch * j + buffer_slice_pitch * k,
-              region[0]);
+      {
+        char const *__restrict h_ptr = adjusted_host_ptr + host_row_pitch * j 
+          + host_slice_pitch * k;       
+        size_t offset = base_offset + buffer_row_pitch * j 
+          + buffer_slice_pitch * k;
+        pocl_tce_write (data, h_ptr, device_ptr, offset, region[0]);
+      }
 }
