@@ -1,6 +1,6 @@
 /* ttasim.cc - a pocl device driver for simulating TTA devices using TCE's ttasim
 
-   Copyright (c) 2012 Pekka Jääskeläinen / Tampere University of Technology
+   Copyright (c) 2012-2015 Pekka Jääskeläinen / Tampere University of Technology
    
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -30,10 +30,15 @@
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <pthread.h>
 #include <string>
 
+#ifndef _MSC_VER
+#  include <unistd.h>
+#else
+#  include "vccompat.hpp"
+#endif
+
 /* Supress some warnings because of including tce_config.h after pocl's config.h. */
 #undef PACKAGE
 #undef PACKAGE_BUGREPORT
@@ -61,8 +66,6 @@
 
 using namespace TTAMachine;
 
-size_t pocl_ttasim_max_work_item_sizes[] = {CL_INT_MAX, CL_INT_MAX, CL_INT_MAX};
-
 static void *pocl_ttasim_thread (void *p);
 
 void
@@ -87,6 +90,7 @@ pocl_ttasim_init_device_ops(struct pocl_device_ops *ops)
   ops->run = pocl_tce_run;
   ops->get_timer_value = pocl_ttasim_get_timer_value;
   ops->init_build = pocl_tce_init_build;
+  ops->build_hash = pocl_tce_build_hash;
 }
 
 
@@ -96,10 +100,8 @@ pocl_ttasim_init_device_infos(struct _cl_device_id* dev)
   dev->type = CL_DEVICE_TYPE_GPU;
   dev->max_compute_units = 1;
   dev->max_work_item_dimensions = 3;
-  dev->max_work_item_sizes[0] = CL_INT_MAX;
-  dev->max_work_item_sizes[1] = CL_INT_MAX;
-  dev->max_work_item_sizes[2] = CL_INT_MAX;
-  dev->max_work_group_size = 8192;
+  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1] =
+	  dev->max_work_item_sizes[2] = dev->max_work_group_size = 8192;
   dev->preferred_wg_size_multiple = 8;
   dev->preferred_vector_width_char = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_CHAR;
   dev->preferred_vector_width_short = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_SHORT;
@@ -127,13 +129,26 @@ pocl_ttasim_init_device_infos(struct _cl_device_id* dev)
   dev->endian_little = CL_FALSE;
   dev->available = CL_TRUE;
   dev->compiler_available = CL_TRUE;
+  dev->spmd = CL_FALSE;
   dev->execution_capabilities = CL_EXEC_KERNEL;
   dev->queue_properties = CL_QUEUE_PROFILING_ENABLE;
   dev->vendor = "TTA-Based Co-design Environment";
   dev->profile = "EMBEDDED_PROFILE";
-  dev->extensions = "";
+  dev->extensions = TCE_DEVICE_EXTENSIONS;
   dev->llvm_target_triplet = "tce-tut-llvm";
   dev->has_64bit_long = 1;
+
+  SETUP_DEVICE_CL_VERSION(TCE_DEVICE_CL_VERSION_MAJOR, TCE_DEVICE_CL_VERSION_MINOR);
+
+  dev->parent_device = NULL;
+  // ttasim does not support partitioning
+  dev->max_sub_devices = 1;
+  dev->num_partition_properties = 1;
+  dev->partition_properties = (cl_device_partition_property*)calloc(dev->num_partition_properties,
+    sizeof(cl_device_partition_property));
+  dev->num_partition_types = 0;
+  dev->partition_type = NULL;
+
 }
 
 unsigned int
@@ -157,7 +172,7 @@ public:
     char dev_name[256];
 
     const char *adf = strrchr(adfName, '/');
-    if (adf != NULL && *adf != NULL) adf++;
+    if (adf != NULL) adf++;
     if (snprintf (dev_name, 256, "ttasim-%s", adf) < 0)
       POCL_ABORT("Unable to generate the device name string.");
     dev->long_name = strdup(dev_name);  
@@ -263,7 +278,7 @@ public:
        the original one. */
 
     /* Create the global buffers along with their initialization data. */
-    for (int i = 0; i < run_cmd->kernel->num_args; ++i)
+    for (size_t i = 0; i < run_cmd->kernel->num_args; ++i)
       {
         struct pocl_argument *al = &(run_cmd->arguments[i]);
         if (run_cmd->kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER)
@@ -338,7 +353,7 @@ public:
     out << "void initialize_kernel_launch() {" << std::endl;
 
     out << "\tkernel_command.kernel = (uint32_t)&" << kernelMdSymbolName << ";" << std::endl;
-    int a = 0;
+    size_t a = 0;
     for (; a < run_cmd->kernel->num_args + run_cmd->kernel->num_locals; ++a)
       {
         struct pocl_argument *al = &(run_cmd->arguments[a]);
