@@ -28,28 +28,24 @@
 #include "config.h"
 #include <assert.h>
 #include <stdio.h>
-#include <ltdl.h>
+#ifndef _MSC_VER
+#  include <ltdl.h>
+#else
+#  include "vccompat.hpp"
+#endif
 #include <pthread.h>
+#ifdef HAVE_CLOCK_GETTIME
+#include <time.h>
+#endif
 
 #define CL_USE_DEPRECATED_OPENCL_1_1_APIS
 #ifdef BUILD_ICD
 #  include "pocl_icd.h"
 #endif
 #include "pocl.h"
+#include "pocl_debug.h"
+#include "pocl_hash.h"
 
-#define POCL_FILENAME_LENGTH 1024
-
-#define POCL_BUILD "pocl-build"
-#define POCL_KERNEL "pocl-kernel"
-#define POCL_WORKGROUP "pocl-workgroup"
-
-/* The filename in which the program source is stored in the program's temp dir. */
-#define POCL_PROGRAM_CL_FILENAME "program.cl"
-/* The filename in which the program LLVM bc is stored in the program's temp dir. */
-#define POCL_PROGRAM_BC_FILENAME "program.bc"
-/* The filename in which the work group (parallelizable) kernel LLVM bc is stored in 
-   the kernel's temp dir. */
-#define POCL_PARALLEL_BC_FILENAME "parallel.bc"
 
 #if __STDC_VERSION__ < 199901L
 # if __GNUC__ >= 2
@@ -65,34 +61,6 @@
 #define LINK_CMD CLANG
 #endif
 
-/* Debugging macros. Also macros for marking unimplemented parts of specs or
-   untested parts of the implementation. */
-
-#define POCL_ABORT_UNIMPLEMENTED()                                      \
-    do {                                                                \
-        fprintf(stderr, "pocl error: encountered unimplemented part of the OpenCL specs in %s:%d\n", __FILE__, __LINE__); \
-        exit(2);                                                        \
-    } while (0) 
-
-#define POCL_WARN_UNTESTED()                                            \
-    do {                                                                \
-        fprintf(stderr, "pocl warning: encountered untested part of the implementation in %s:%d\n", __FILE__, __LINE__); \
-    } while (0) 
-
-#define POCL_WARN_INCOMPLETE()                                            \
-    do {                                                                \
-        fprintf(stderr, "pocl warning: encountered incomplete implementation in %s:%d\n", __FILE__, __LINE__); \
-    } while (0) 
-
-#define POCL_ABORT(__MSG__)                                      \
-    do {                                                                \
-        fprintf(stderr, __MSG__); \
-        exit(2);                                                        \
-    } while (0) 
-
-#define POCL_ERROR(x) do { if (errcode_ret != NULL) {*errcode_ret = (x); } return NULL; } while (0)
-#define POCL_SUCCESS() do { if (errcode_ret != NULL) {*errcode_ret = CL_SUCCESS; } } while (0)
-
 typedef pthread_mutex_t pocl_lock_t;
 #define POCL_LOCK_INITIALIZER PTHREAD_MUTEX_INITIALIZER
 
@@ -130,6 +98,12 @@ typedef pthread_mutex_t pocl_lock_t;
     (__OBJ__)->pocl_refcount = 1;                  \
   } while (0)
 
+#define POCL_MEM_FREE(F_PTR)                      \
+  do {                                            \
+      free((F_PTR));                              \
+      (F_PTR) = NULL;                             \
+  } while (0)
+
 #ifdef BUILD_ICD
 /* Most (all?) object must also initialize the ICD field */
 #  define POCL_INIT_OBJECT(__OBJ__)                \
@@ -161,6 +135,14 @@ typedef pthread_mutex_t pocl_lock_t;
 #  define POsym(name)
 #  define POsymAlways(name)
 
+#elif defined(_MSC_VER)
+/* Visual Studio does not support this magic either */
+#  define POname(name) name
+#  define POdeclsym(name)
+#  define POsym(name)
+#  define POsymAlways(name)
+#  define POdeclsym(name)
+
 #else
 /* Symbol aliases are supported */
 
@@ -226,7 +208,7 @@ struct pocl_argument_info {
 };
 
 struct pocl_device_ops {
-  char *device_name;
+  const char *device_name;
   void (*init_device_infos) (struct _cl_device_id*);
   /* implementation */
   void (*uninit) (cl_device_id device);
@@ -234,8 +216,10 @@ struct pocl_device_ops {
   void (*init) (cl_device_id device, const char *parameters);
   cl_int (*alloc_mem_obj) (cl_device_id device, cl_mem mem_obj);
   void *(*create_sub_buffer) (void *data, void* buffer, size_t origin, size_t size);
-  void (*free) (void *data, cl_mem_flags flags, void *ptr);
-  void (*read) (void *data, void *host_ptr, const void *device_ptr, size_t cb);
+  void (*free) (cl_device_id device, cl_mem mem_obj);
+  void (*free_ptr) (cl_device_id device, void* mem_ptr);
+  void (*read) (void *data, void *host_ptr, const void *device_ptr,
+                size_t offset, size_t cb);
   void (*read_rect) (void *data, void *host_ptr, void *device_ptr,
                      const size_t *buffer_origin,
                      const size_t *host_origin, 
@@ -244,7 +228,8 @@ struct pocl_device_ops {
                      size_t buffer_slice_pitch,
                      size_t host_row_pitch,
                      size_t host_slice_pitch);
-  void (*write) (void *data, const void *host_ptr, void *device_ptr, size_t cb);
+  void (*write) (void *data, const void *host_ptr, void *device_ptr, 
+                 size_t offset, size_t cb);
   void (*write_rect) (void *data, const void *host_ptr, void *device_ptr,
                       const size_t *buffer_origin,
                       const size_t *host_origin, 
@@ -253,7 +238,8 @@ struct pocl_device_ops {
                       size_t buffer_slice_pitch,
                       size_t host_row_pitch,
                       size_t host_slice_pitch);
-  void (*copy) (void *data, const void *src_ptr,  void *__restrict__ dst_ptr, size_t cb);
+  void (*copy) (void *data, const void *src_ptr, size_t src_offset, 
+                void *__restrict__ dst_ptr, size_t dst_offset, size_t cb);
   void (*copy_rect) (void *data, const void *src_ptr, void *dst_ptr,
                      const size_t *src_origin,
                      const size_t *dst_origin, 
@@ -272,6 +258,12 @@ void (*fill_rect) (void *data,
                    void *fill_pixel,
                    size_t pixel_size);
 
+void (*memfill) (void *ptr,
+                 size_t size,
+                 size_t offset,
+                 const void* pattern,
+                 size_t pattern_size);
+
   /* Maps 'size' bytes of device global memory at buf_ptr + offset to 
      host-accessible memory. This might or might not involve copying 
      the block from the device. */
@@ -287,16 +279,22 @@ void (*fill_rect) (void *data,
   /* Perform initialization steps and can return additional
      build options that are required for the device. The caller
      owns the returned string. */
-  char* (*init_build) 
-  (void *data, 
-   const char *dev_tmpdir);
+  char* (*init_build) (void *data);
+
+  void (*build_hash) (void *data, SHA1_CTX *build_hash);
 
     /* return supported image formats */
   cl_int (*get_supported_image_formats) (cl_mem_flags flags,
                                          const cl_image_format **image_formats,
-                                         cl_int *num_image_formats);
+                                         cl_uint *num_image_formats);
 };
 
+typedef struct pocl_global_mem_t {
+  size_t max_ever_allocated;
+  size_t currently_allocated;
+  size_t total_alloc_limit;
+} pocl_global_mem_t;
+
 struct _cl_device_id {
   POCL_ICD_OBJECT
   POCL_OBJECT;
@@ -346,6 +344,8 @@ struct _cl_device_id {
   cl_uint global_mem_cacheline_size;
   cl_ulong global_mem_cache_size;
   cl_ulong global_mem_size;
+  size_t global_var_pref_size;
+  size_t global_var_max_size;
   cl_ulong max_constant_buffer_size;
   cl_uint max_constant_args;
   cl_device_local_mem_type local_mem_type;
@@ -356,19 +356,33 @@ struct _cl_device_id {
   cl_bool endian_little;
   cl_bool available;
   cl_bool compiler_available;
+  /* Is the target a Single Program Multiple Data machine? If not,
+     we need to generate work-item loops to execute all the work-items
+     in the WG, otherwise the hardware spawns the WIs. */
+  cl_bool spmd;
   cl_device_exec_capabilities execution_capabilities;
   cl_command_queue_properties queue_properties;
   cl_platform_id platform;
-  cl_device_partition_property device_partition_properties[1];
+  cl_uint max_sub_devices;
+  size_t num_partition_properties;
+  cl_device_partition_property *partition_properties;
+  size_t num_partition_types;
+  cl_device_partition_property *partition_type;
   size_t printf_buffer_size;
   char *short_name;
   char *long_name;
+  char *cache_dir_name;
+  cl_device_id parent_device;
 
-  char *vendor;
-  char *driver_version;
-  char *profile;
-  char *version;
-  char *extensions;
+  const char *vendor;
+  const char *driver_version;
+  const char *profile;
+  const char *version;
+  const char *extensions;
+  cl_uint cl_version_major;    // 2
+  cl_uint cl_version_minor;    // 0
+  const char *cl_version_std;  // "CL2.0"
+  cl_uint cl_version_int;      // 200
  
   void *data;
   const char* llvm_target_triplet; /* the llvm target triplet to use */
@@ -377,11 +391,38 @@ struct _cl_device_id {
      indexing  arrays in data structures with device specific entries. */
   int dev_id;
   int global_mem_id; /* identifier for device global memory */
+  /* pointer to an accounting struct for global memory */
+  pocl_global_mem_t *global_memory;
   int has_64bit_long;  /* Does the device have 64bit longs */
+  /* Convert automatic local variables to kernel arguments? */
+  int autolocals_to_args;
+
+  // True if the device supports SVM has priority
+  // at allocating Shared Virtual Memory
+  cl_bool should_allocate_svm;
+  /* OpenCL 2.0 properties */
+  cl_device_svm_capabilities svm_caps;
+  cl_uint max_events;
+  cl_uint max_queues;
+  cl_uint max_pipe_args;
+  cl_uint max_pipe_active_res;
+  cl_uint max_pipe_packet_size;
+  cl_uint dev_queue_pref_size;
+  cl_uint dev_queue_max_size;
+  cl_command_queue_properties on_dev_queue_props;
+  cl_command_queue_properties on_host_queue_props;
 
   struct pocl_device_ops *ops; /* Device operations, shared amongst same devices */
 };
 
+#define DEVICE_SVM_FINEGR(dev) (dev->svm_caps & (CL_DEVICE_SVM_FINE_GRAIN_BUFFER \
+                                              | CL_DEVICE_SVM_FINE_GRAIN_SYSTEM))
+#define DEVICE_SVM_ATOM(dev) (dev->svm_caps & CL_DEVICE_SVM_ATOMICS)
+
+#define DEVICE_IS_SVM_CAPABLE(dev) (dev->svm_caps & CL_DEVICE_SVM_COARSE_GRAIN_BUFFER)
+
+#define DEVICE_MMAP_IS_NOP(dev) (DEVICE_SVM_FINEGR(dev) && DEVICE_SVM_ATOM(dev))
+
 struct _cl_platform_id {
   POCL_ICD_OBJECT
 }; 
@@ -400,6 +441,13 @@ struct _cl_context {
      clReleaseContext for the result regardless if it failed or not. 
      Returns a valid = 0 context in that case.  */
   char valid;
+
+  /* The minimal value of max_mem_alloc_size of all devices in context */
+  size_t min_max_mem_alloc_size;
+  /* The device that should allocate SVM (might be == host)
+   * NULL if none of devices in the context is SVM capable */
+  cl_device_id svm_allocdev;
+
 };
 
 struct _cl_command_queue {
@@ -462,8 +510,13 @@ struct _cl_mem {
   cl_uint                 num_mip_levels;
   cl_uint                 num_samples;
   cl_mem                  buffer;
+  /* Pipe specific */
+  cl_uint packet_size;
+  cl_uint max_packets;
 };
 
+typedef uint8_t SHA1_digest_t[SHA1_DIGEST_SIZE * 2 + 1];
+
 struct _cl_program {
   POCL_ICD_OBJECT
   POCL_OBJECT;
@@ -479,19 +532,26 @@ struct _cl_program {
      sequential bitcode produced from the kernel sources.*/
   size_t *binary_sizes; 
   unsigned char **binaries; 
-  /* Temp directory (relative to CWD) where the kernel files reside. */
-  char *temp_dir;
   /* implementation */
   cl_kernel kernels;
+  /* Per-device program hash after build */
+  SHA1_digest_t* build_hash;
+  /* Per-device build logs, for the case when we don't yet have the program's cachedir */
+  char** build_log;
+  /* Per-program build log, for the case when we aren't yet building for devices */
+  char main_build_log[640];
   /* Used to store the llvm IR of the build to save disk I/O. */
   void **llvm_irs;
+  /* Read locks */
+  void** read_locks;
+  /* Use to store build status */
+  cl_build_status build_status;
 };
 
 struct _cl_kernel {
   POCL_ICD_OBJECT
   POCL_OBJECT;
   /* queries */
-  char *function_name;
   char *name;
   cl_uint num_args;
   cl_context context;
@@ -514,13 +574,14 @@ struct event_callback_item
   void (*callback_function) (cl_event, cl_int, void*);
   void *user_data;
   cl_int trigger_status;
-  void *next;
+  struct event_callback_item *next;
 };
 
 typedef struct _cl_event _cl_event;
 struct _cl_event {
   POCL_ICD_OBJECT
   POCL_OBJECT;
+  cl_context context;
   cl_command_queue queue;
   cl_command_type command_type;
 
@@ -550,54 +611,58 @@ struct _cl_sampler {
   cl_filter_mode      filter_mode;
 };
 
-#define POCL_UPDATE_EVENT_QUEUED(__event, __cq)                         \
+#define POCL_UPDATE_EVENT_QUEUED(__event)                               \
   do {                                                                  \
     if ((__event) != NULL && (*(__event)) != NULL)                      \
       {                                                                 \
+        cl_command_queue __cq = (*(__event))->queue;                    \
         (*(__event))->status = CL_QUEUED;                               \
-        if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
+        if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
           (*(__event))->time_queue =                                    \
-            (__cq)->device->ops->get_timer_value((__cq)->device->data);      \
+            __cq->device->ops->get_timer_value(__cq->device->data);     \
       }                                                                 \
   } while (0)                                                           \
 
-#define POCL_UPDATE_EVENT_SUBMITTED(__event, __cq)                      \
+#define POCL_UPDATE_EVENT_SUBMITTED(__event)                            \
   do {                                                                  \
     if ((__event) != NULL && (*(__event)) != NULL)                      \
       {                                                                 \
         assert((*(__event))->status == CL_QUEUED);                      \
         (*(__event))->status = CL_SUBMITTED;                            \
-        if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
+        cl_command_queue __cq = (*(__event))->queue;                    \
+        if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
           (*(__event))->time_submit =                                   \
-            (__cq)->device->ops->get_timer_value((__cq)->device->data);      \
+            __cq->device->ops->get_timer_value(__cq->device->data);     \
       }                                                                 \
   } while (0)                                                           \
 
-#define POCL_UPDATE_EVENT_RUNNING(__event, __cq)                        \
+#define POCL_UPDATE_EVENT_RUNNING(__event)                              \
   do {                                                                  \
     if (__event != NULL && (*(__event)) != NULL)                        \
       {                                                                 \
         assert((*(__event))->status == CL_SUBMITTED);                   \
         (*(__event))->status = CL_RUNNING;                              \
-        if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
+        cl_command_queue __cq = (*(__event))->queue;                    \
+        if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
           (*(__event))->time_start =                                    \
-            (__cq)->device->ops->get_timer_value((__cq)->device->data);      \
+            __cq->device->ops->get_timer_value(__cq->device->data);     \
       }                                                                 \
   } while (0)                                                           \
 
-#define POCL_UPDATE_EVENT_COMPLETE(__event, __cq)                       \
+#define POCL_UPDATE_EVENT_COMPLETE(__event)                             \
   do {                                                                  \
     if ((__event) != NULL && (*(__event)) != NULL)                      \
       {                                                                 \
         assert((*(__event))->status == CL_RUNNING);                     \
         (*(__event))->status = CL_COMPLETE;                             \
-        if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
+        cl_command_queue __cq = (*(__event))->queue;                    \
+        if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
           (*(__event))->time_end =                                      \
-            (__cq)->device->ops->get_timer_value((__cq)->device->data);      \
+            __cq->device->ops->get_timer_value(__cq->device->data);     \
       }                                                                 \
   } while (0)                                                           \
 
 #define min(a,b) (((a) < (b)) ? (a) : (b))
 #define max(a,b) (((a) > (b)) ? (a) : (b))
-    
+
 #endif /* POCL_CL_H */
