@@ -29,16 +29,15 @@
 extern "C" {
 #endif
 
+/* Returns the cpu name as reported by LLVM. */
+char* get_cpu_name();
+
 /* Compiles an .cl file into LLVM IR.
  */
 int pocl_llvm_build_program
 (cl_program program,
- cl_device_id device,
- int device_i,     
- const char* temp_dir,
- const char* binary_filename,
- const char* device_tmpdir,
- const char* user_options);
+ unsigned device_i,
+ const char* user_options_cstr, char *program_bc_path);
 
 
 /* Retrieve metadata of the given kernel in the program to populate the
@@ -49,8 +48,6 @@ int pocl_llvm_get_kernel_metadata
  cl_kernel kernel,
  int device_i,     
  const char* kernel_name,
- const char* device_tmpdir, 
- char* descriptor_filename,
  int *errcode);
 
 /* This function links the input kernel LLVM bitcode and the
@@ -66,11 +63,14 @@ int pocl_llvm_get_kernel_metadata
  * at a time or control the options through thread safe methods.
  */
 int pocl_llvm_generate_workgroup_function
-(cl_device_id device,
- cl_kernel kernel,
- size_t local_x, size_t local_y, size_t local_z,
- const char* parallel_filename,
- const char* kernel_filename);
+(cl_device_id device, cl_kernel kernel,
+ size_t local_x, size_t local_y, size_t local_z
+);
+
+/**
+ * Free the LLVM IR of a program for a given device
+ */
+void pocl_free_llvm_irs(cl_program program, int device_i);
 
 /**
  * Update the program->binaries[] representation of the kernels
@@ -79,6 +79,13 @@ int pocl_llvm_generate_workgroup_function
  */
 void pocl_llvm_update_binaries (cl_program program);
 
+/**
+ * Count the number of "__kernel" functions in 'program'.
+ *
+ * Results are valid as long as program binary is not modified.
+ */
+unsigned pocl_llvm_get_kernel_count(cl_program program);
+
 /**
  * Find the "__kernel" function names in 'program',
  * filling the callee-allocated array with pointer to the program binary.
@@ -89,7 +96,7 @@ void pocl_llvm_update_binaries (cl_program program);
  * Returns the number of kernels found in the program (may be greater than
  * 'max_num_krn')
  */
-int pocl_llvm_get_kernel_names( cl_program program, const char **knames, unsigned max_num_krn);
+unsigned pocl_llvm_get_kernel_names( cl_program program, const char **knames, unsigned max_num_krn);
 
 /** Compile the kernel in infile from LLVM bitcode to native object file for
  * device, into outfile.
@@ -99,7 +106,14 @@ int pocl_llvm_codegen ( cl_kernel kernel,
                         const char *infile,
                         const char *outfile);
 
+/* Parse program file and populate program's llvm_irs */
+int
+pocl_update_program_llvm_irs(cl_program program, unsigned device_i,
+                             cl_device_id device);
+
+
+
+
 #ifdef __cplusplus
 }
 #endif
-
