@@ -21,27 +21,25 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "config.h"
+#include <iostream>
+
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include "pocl.h"
+
+#include "llvm/Transforms/Utils/BasicBlockUtils.h"
+#include "llvm/IR/Instructions.h"
+#include "llvm/IR/Module.h"
+#include "llvm/IR/Dominators.h"
+
 #include "CanonicalizeBarriers.h"
 #include "BarrierBlock.h"
 #include "Barrier.h"
 #include "Workgroup.h"
-#include "llvm/Transforms/Utils/BasicBlockUtils.h"
-#include <iostream>
-
 #include "VariableUniformityAnalysis.h"
 
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#else
-#include "llvm/IR/Instructions.h"
-#include "llvm/IR/Module.h"
-#endif
-
-#if not (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-#include "llvm/IR/Dominators.h"
-#endif
+POP_COMPILER_DIAGS
 
 using namespace llvm;
 using namespace pocl;
@@ -57,11 +55,7 @@ char CanonicalizeBarriers::ID = 0;
 void
 CanonicalizeBarriers::getAnalysisUsage(AnalysisUsage &AU) const
 {
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  AU.addRequired<DominatorTree>();
-#else
   AU.addRequired<DominatorTreeWrapperPass>();
-#endif
   AU.addPreserved<VariableUniformityAnalysis>();    
 }
 
@@ -73,25 +67,31 @@ CanonicalizeBarriers::runOnFunction(Function &F)
 
   BasicBlock *entry = &F.getEntryBlock();
   if (!isa<BarrierBlock>(entry)) {
+#ifdef LLVM_OLDER_THAN_3_7
     BasicBlock *effective_entry = SplitBlock(entry, 
                                              &(entry->front()),
                                              this);
+#else
+    BasicBlock *effective_entry = SplitBlock(entry, 
+                                             &(entry->front()));
+#endif
+
     effective_entry->takeName(entry);
     entry->setName("entry.barrier");
     Barrier::Create(entry->getTerminator());
   }
 
   for (Function::iterator i = F.begin(), e = F.end(); i != e; ++i) {
-    BasicBlock *b = i;
+    BasicBlock *b = &*i;
     TerminatorInst *t = b->getTerminator();
 
     const bool isExitNode = 
       (t->getNumSuccessors() == 0) && (!isa<BarrierBlock>(b));
 
     // The function exits should have barriers.
-    if (isExitNode) {
+    if (isExitNode && !Barrier::hasOnlyBarrier(b)) {
       /* In case the bb is already terminated with a barrier,
-         split before the barrier so we dot create an empty
+         split before the barrier so we don't create an empty
          parallel region.
          
          This is because the assumptions of the other passes in the 
@@ -101,34 +101,24 @@ CanonicalizeBarriers::runOnFunction(Function &F)
          between the explicit barrier and the added one). */
       BasicBlock *exit; 
       if (Barrier::endsWithBarrier(b))
+#ifdef LLVM_OLDER_THAN_3_7
         exit = SplitBlock(b, t->getPrevNode(), this);
+#else
+        exit = SplitBlock(b, t->getPrevNode());
+#endif
       else
+#ifdef LLVM_OLDER_THAN_3_7
         exit = SplitBlock(b, t, this);
+#else
+        exit = SplitBlock(b, t);
+#endif
       exit->setName("exit.barrier");
       Barrier::Create(t);
     }
   }
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  DT = getAnalysisIfAvailable<DominatorTree>();
-#else
   DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
-#endif
-  LI = getAnalysisIfAvailable<LoopInfo>();
-
-  bool changed = ProcessFunction(F);
-
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  if (DT)
-    DT->verifyAnalysis();
-#else
-  if (DT)
-    DT->verifyDomTree();
-#endif
-  if (LI)
-    LI->verifyAnalysis();
-
-  return changed;
+  return ProcessFunction(F);
 }
 
 
@@ -145,11 +135,11 @@ CanonicalizeBarriers::ProcessFunction(Function &F) {
 
   for (Function::iterator i = F.begin(), e = F.end();
        i != e; ++i) {
-    BasicBlock *b = i;
+    BasicBlock *b = &*i;
     for (BasicBlock::iterator i = b->begin(), e = b->end();
          i != e; ++i) {
       if (isa<Barrier>(i)) {
-        Barriers.insert(i);
+        Barriers.insert(&*i);
       }
     }
   }
@@ -173,7 +163,11 @@ CanonicalizeBarriers::ProcessFunction(Function &F) {
       t->getPrevNode() != *i;
 
     if (HAS_NON_BRANCH_INSTRUCTIONS_AFTER_BARRIER) {
+#ifdef LLVM_OLDER_THAN_3_7
       BasicBlock *new_b = SplitBlock(b, (*i)->getNextNode(), this);
+#else
+      BasicBlock *new_b = SplitBlock(b, (*i)->getNextNode());
+#endif
       new_b->setName(b->getName() + ".postbarrier");
       changed = true;
     }
@@ -198,7 +192,11 @@ CanonicalizeBarriers::ProcessFunction(Function &F) {
     // (allow multiple predecessors, eases loop handling).
     // if (&b->front() == (*i))
     //   continue;
+#ifdef LLVM_OLDER_THAN_3_7
     BasicBlock *new_b = SplitBlock(b, *i, this);
+#else
+    BasicBlock *new_b = SplitBlock(b, *i);
+#endif
     new_b->takeName(b);
     b->setName(new_b->getName() + ".prebarrier");
     changed = true;
@@ -211,7 +209,7 @@ CanonicalizeBarriers::ProcessFunction(Function &F) {
     emptyRegionDeleted = false;
     for (Function::iterator i = F.begin(), e = F.end();
          i != e; ++i) {
-        BasicBlock *b = i;
+        BasicBlock *b = &*i;
         llvm::TerminatorInst *t = b->getTerminator();
         if (!Barrier::endsWithBarrier(b) || t->getNumSuccessors() != 1) 
           continue;
