@@ -21,28 +21,21 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+#include <iostream>
+
 #include "config.h"
-#include "ImplicitLoopBarriers.h"
-#include "Barrier.h"
-#include "Workgroup.h"
+
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-#include "llvm/Constants.h"
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#else
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Module.h"
-#endif
-#if not (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
 #include "llvm/IR/Dominators.h"
-#endif
 
+#include "ImplicitLoopBarriers.h"
+#include "Barrier.h"
+#include "Workgroup.h"
 #include "VariableUniformityAnalysis.h"
 
-#include <iostream>
-
 //#define DEBUG_ILOOP_BARRIERS
 
 using namespace llvm;
@@ -57,13 +50,8 @@ namespace {
 char ImplicitLoopBarriers::ID = 0;
 
 void ImplicitLoopBarriers::getAnalysisUsage(AnalysisUsage &AU) const {
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  AU.addRequired<DominatorTree>();
-  AU.addPreserved<DominatorTree>();
-#else
   AU.addRequired<DominatorTreeWrapperPass>();
   AU.addPreserved<DominatorTreeWrapperPass>();
-#endif
   AU.addRequired<VariableUniformityAnalysis>();
   AU.addPreserved<VariableUniformityAnalysis>();
 }
@@ -72,6 +60,14 @@ bool ImplicitLoopBarriers::runOnLoop(Loop *L, LPPassManager &LPM) {
   if (!Workgroup::isKernelToProcess(*L->getHeader()->getParent()))
     return false;
 
+  if (!Workgroup::hasWorkgroupBarriers(*L->getHeader()->getParent())) {
+#ifdef DEBUG_ILOOP_BARRIERS
+    std::cerr << "### ILB: The kernel has no barriers, let's not add implicit ones "
+              << "either to avoid WI context switch overheads"
+              << std::endl;
+#endif
+    return false;
+  }
   return ProcessLoop(L, LPM);
 }
 
