@@ -23,15 +23,9 @@
 #include "LLVMUtils.h"
 
 #include "pocl.h"
-#include "config.h"
 
-#ifdef LLVM_3_2
-#include <llvm/Module.h>
-#include <llvm/Metadata.h>
-#else
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Metadata.h>
-#endif
 
 using namespace llvm;
 
@@ -60,17 +54,19 @@ regenerate_kernel_metadata(llvm::Module &M, FunctionMapping &kernels)
             {
               Function *old_kernel = (*i).first;
               Function *new_kernel = (*i).second;
+              Function *func_from_md;
+              func_from_md = dyn_cast<Function>(
+                dyn_cast<ValueAsMetadata>(wgsizeMD->getOperand(0))->getValue());
               if (old_kernel == new_kernel || wgsizeMD->getNumOperands() == 0 ||
-                  dyn_cast<Function>(wgsizeMD->getOperand(0)) != old_kernel) 
+                  func_from_md != old_kernel) 
                 continue;
               // found a wg size metadata that points to the old kernel, copy its
               // operands except the first one to a new MDNode
-              SmallVector<Value*, 8> operands;
-              operands.push_back(new_kernel);
-              for (unsigned opr = 1; opr < wgsizeMD->getNumOperands(); ++opr)
-                {
+              SmallVector<Metadata*, 8> operands;
+              operands.push_back(llvm::ValueAsMetadata::get(new_kernel));
+              for (unsigned opr = 1; opr < wgsizeMD->getNumOperands(); ++opr) {
                   operands.push_back(wgsizeMD->getOperand(opr));
-                }
+              }
               MDNode *new_wg_md = MDNode::get(M.getContext(), operands);
               wg_sizes->addOperand(new_wg_md);
             } 
@@ -86,7 +82,8 @@ regenerate_kernel_metadata(llvm::Module &M, FunctionMapping &kernels)
   for (FunctionMapping::const_iterator i = kernels.begin(),
          e = kernels.end();
        i != e; ++i) {
-    MDNode *md = MDNode::get(M.getContext(), ArrayRef<Value *>((*i).second));
+    MDNode *md = MDNode::get(M.getContext(), ArrayRef<Metadata *>(
+      llvm::ValueAsMetadata::get((*i).second)));
     nmd->addOperand(md);
   }
 }
