@@ -2,7 +2,7 @@
 // each kernel should run in parallel.
 // 
 // Copyright (c) 2011 Universidad Rey Juan Carlos and
-//               2012-2014 Pekka Jääskeläinen / TUT
+//               2012-2015 Pekka Jääskeläinen / TUT
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -22,28 +22,23 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "ParallelRegion.h"
-#include "Barrier.h"
-#include "Kernel.h"
-#include "config.h"
-#ifdef LLVM_3_1
-#include "llvm/Support/IRBuilder.h"
-#include "llvm/ValueSymbolTable.h"
-#elif defined LLVM_3_2
-#include "llvm/IRBuilder.h"
-#include "llvm/ValueSymbolTable.h"
-#else
-#include "llvm/IR/IRBuilder.h"
-#include "llvm/IR/ValueSymbolTable.h"
-#endif
-#include "llvm/Transforms/Utils/BasicBlockUtils.h"
-#include "llvm/Transforms/Utils/Cloning.h"
-
 #include <set>
 #include <sstream>
 #include <map>
 #include <algorithm>
 
+#include "pocl.h"
+
+#include "llvm/IR/IRBuilder.h"
+#include "llvm/IR/ValueSymbolTable.h"
+#include "llvm/Transforms/Utils/BasicBlockUtils.h"
+#include "llvm/Transforms/Utils/Cloning.h"
+
+#include "ParallelRegion.h"
+#include "Barrier.h"
+#include "Kernel.h"
+#include "DebugHelpers.h"
+
 using namespace std;
 using namespace llvm;
 using namespace pocl;
@@ -75,7 +70,7 @@ ParallelRegion::GenerateTempNames(llvm::BasicBlock *bb)
 {
   for (llvm::BasicBlock::iterator i = bb->begin(), e = bb->end(); i != e; ++i)
     {
-      llvm::Instruction *instr = i;
+      llvm::Instruction *instr = &*i;
       if (instr->hasName() || !instr->isUsedOutsideOfBlock(bb)) continue;
       int tempCounter = 0;
       std::string tempName = "";
@@ -165,7 +160,7 @@ ParallelRegion::remap(ValueToValueMapTy &map)
 
     for (BasicBlock::iterator ii = (*i)->begin(), ee = (*i)->end();
          ii != ee; ++ii)
-      RemapInstruction(ii, map,
+      RemapInstruction(&*ii, map,
                        RF_IgnoreMissingEntries | RF_NoModuleLevelChanges);
 
 #ifdef DEBUG_REMAP
@@ -209,8 +204,12 @@ ParallelRegion::chainAfter(ParallelRegion *region)
     successor->getParent()->getBasicBlockList();
   
   for (iterator i = begin(), e = end(); i != e; ++i)
+
+#ifdef LLVM_OLDER_THAN_3_8
     bb_list.insertAfter(tail, *i);
-  
+#else
+    bb_list.insertAfter(tail->getIterator(), *i);
+#endif
   t->setSuccessor(0, entryBB());
 
   t = exitBB()->getTerminator();
@@ -293,11 +292,13 @@ ParallelRegion::insertLocalIdInit(llvm::BasicBlock* entry,
   Module *M = entry->getParent()->getParent();
 
   int size_t_width = 32;
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  if (M->getPointerSize() == llvm::Module::Pointer64)
-#else
-  // FIXME 0 here is the address space: this breaks (?) if _local_size_x is not stored in AS0
+#ifdef LLVM_OLDER_THAN_3_7
+  // This breaks (?) if _local_size_x is not stored in AS0,
+  // but it always will be as it's just a pseudo variable that
+  // will be scalarized.
   if (M->getDataLayout()->getPointerSize(0) == 8)
+#else
+  if (M->getDataLayout().getPointerSize(0) == 8)
 #endif
     size_t_width = 64;
 
@@ -363,10 +364,10 @@ ParallelRegion::Create(const SmallPtrSet<BasicBlock *, 8>& bbs, BasicBlock *entr
   // is the same as original function order.
   Function *F = entry->getParent();
   for (Function::iterator i = F->begin(), e = F->end(); i != e; ++i) {
-    BasicBlock *b = i;
+    BasicBlock *b = &*i;
     for (SmallPtrSetIterator<BasicBlock *> j = bbs.begin(); j != bbs.end(); ++j) {
       if (*j == b) {
-        new_region->push_back(i);
+        new_region->push_back(&*i);
         if (entry == *j)
             new_region->setEntryBBIndex(new_region->size() - 1);
         else if (exit == *j)
@@ -402,9 +403,14 @@ ParallelRegion::Verify()
           std::cerr << "suspicious block: " << (*i)->getName().str() << std::endl;
           std::cerr << "the entry is: " << entryBB()->getName().str() << std::endl;
 
-#if 0
-          (*i)->getParent()->viewCFG();
-#endif
+          ParallelRegion::ParallelRegionVector prvec;
+          prvec.push_back(this);
+          std::set<llvm::BasicBlock*> highlights;
+          highlights.insert(entryBB());
+          highlights.insert(*i);
+          pocl::dumpCFG(
+            *(*i)->getParent(), (*i)->getParent()->getName().str() + ".dot",
+            &prvec, &highlights);
           assert(0 && "Incoming edges to non-entry block!");
           return false;
         } else if (!Barrier::hasBarrier(*ii)) {
@@ -420,8 +426,17 @@ ParallelRegion::Verify()
     //   assert(0 && "Parallel regions must be single entry!");
     //   return false;
     // }
-
     if (exitBB()->getTerminator()->getNumSuccessors() != 1) {
+      ParallelRegion::ParallelRegionVector regions;
+      regions.push_back(this);
+
+      std::set<llvm::BasicBlock*> highlights;
+      highlights.insert((*i));
+      highlights.insert(exitBB());
+      exitBB()->dump();
+      dumpNames();
+      dumpCFG(*(*i)->getParent(), "broken.dot", &regions, &highlights);
+
       assert(0 && "Multiple outgoing edges from exit block!");
       return false;
     }
@@ -460,22 +475,18 @@ ParallelRegion::Verify()
  */
 void
 ParallelRegion::AddParallelLoopMetadata(llvm::MDNode *identifier) {
- 
+
   for (iterator i = begin(), e = end(); i != e; ++i) {
     BasicBlock* bb = *i;      
     for (BasicBlock::iterator ii = bb->begin(), ee = bb->end();
          ii != ee; ii++) {
       if (ii->mayReadOrWriteMemory()) {
-        std::vector<Value*> loopIds;
-        MDNode *oldIds = ii->getMetadata("llvm.mem.parallel_loop_access");
-        if (oldIds != NULL) {
-          for (unsigned i = 0; i < oldIds->getNumOperands(); ++i) {
-            loopIds.push_back(oldIds->getOperand(i));
-          }
+        MDNode *newMD = MDNode::get(bb->getContext(), identifier);
+        MDNode *oldMD = ii->getMetadata("llvm.mem.parallel_loop_access");
+        if (oldMD != NULL) {
+          newMD = llvm::MDNode::concatenate(oldMD, newMD);
         }
-        loopIds.push_back(identifier);
-        ii->setMetadata("llvm.mem.parallel_loop_access", 
-                        MDNode::get(bb->getContext(), loopIds));
+        ii->setMetadata("llvm.mem.parallel_loop_access", newMD);
       }
     }
   }
@@ -487,31 +498,36 @@ ParallelRegion::AddIDMetadata(
     std::size_t x, 
     std::size_t y, 
     std::size_t z) {
-  
     int counter = 1;
-    Value *v1[] = {
+    Metadata *v1[] = {
         MDString::get(context, "WI_region"),      
-        ConstantInt::get(Type::getInt32Ty(context), pRegionId)};      
+        llvm::ConstantAsMetadata::get(
+          ConstantInt::get(Type::getInt32Ty(context), pRegionId))
+    };
     MDNode* mdRegion = MDNode::get(context, v1);  
-    Value *v2[] = {
+    Metadata *v2[] = {
         MDString::get(context, "WI_xyz"),      
-        ConstantInt::get(Type::getInt32Ty(context), x),
-        ConstantInt::get(Type::getInt32Ty(context), y),      
-        ConstantInt::get(Type::getInt32Ty(context), z)};      
+        llvm::ConstantAsMetadata::get(
+          ConstantInt::get(Type::getInt32Ty(context), x)),
+        llvm::ConstantAsMetadata::get(
+          ConstantInt::get(Type::getInt32Ty(context), y)),      
+        llvm::ConstantAsMetadata::get(
+          ConstantInt::get(Type::getInt32Ty(context), z))};
     MDNode* mdXYZ = MDNode::get(context, v2);  
-    Value *v[] = {
+    Metadata *v[] = {
         MDString::get(context, "WI_data"),      
         mdRegion,
         mdXYZ};
     MDNode* md = MDNode::get(context, v);              
     
     for (iterator i = begin(), e = end(); i != e; ++i) {
-      BasicBlock* bb = *i;      
+      BasicBlock* bb = *i;
       for (BasicBlock::iterator ii = bb->begin();
             ii != bb->end(); ii++) {
-        Value *v3[] = {
+        Metadata *v3[] = {
             MDString::get(context, "WI_counter"),      
-            ConstantInt::get(Type::getInt32Ty(context), counter)};      
+            llvm::ConstantAsMetadata::get(
+              ConstantInt::get(Type::getInt32Ty(context), counter))};
         MDNode* mdCounter = MDNode::get(context, v3);  
         counter++;
         ii->setMetadata("wi", md);
@@ -579,7 +595,7 @@ llvm::Instruction*
 ParallelRegion::LocalIDZLoad()
 {
   if (LocalIDZLoadInstr != NULL) return LocalIDZLoadInstr;
-  IRBuilder<> builder(entryBB()->getFirstInsertionPt());
+  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
   return LocalIDZLoadInstr = 
     builder.CreateLoad
     (entryBB()->getParent()->getParent()->getGlobalVariable(POCL_LOCAL_ID_Z_GLOBAL));
@@ -593,7 +609,7 @@ llvm::Instruction*
 ParallelRegion::LocalIDYLoad()
 {
   if (LocalIDYLoadInstr != NULL) return LocalIDYLoadInstr;
-  IRBuilder<> builder(entryBB()->getFirstInsertionPt());
+  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
   return LocalIDYLoadInstr = 
     builder.CreateLoad
     (entryBB()->getParent()->getParent()->getGlobalVariable(POCL_LOCAL_ID_Y_GLOBAL));
@@ -607,7 +623,7 @@ llvm::Instruction*
 ParallelRegion::LocalIDXLoad()
 {
   if (LocalIDXLoadInstr != NULL) return LocalIDXLoadInstr;
-  IRBuilder<> builder(entryBB()->getFirstInsertionPt());
+  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
   return LocalIDXLoadInstr = 
     builder.CreateLoad
     (entryBB()->getParent()->getParent()->getGlobalVariable(POCL_LOCAL_ID_X_GLOBAL));
@@ -615,7 +631,7 @@ ParallelRegion::LocalIDXLoad()
 
 void
 ParallelRegion::InjectPrintF
-(llvm::Instruction *before, std::string formatStr, 
+(llvm::Instruction *before, std::string formatStr,
  std::vector<Value*>& params)
 {
   IRBuilder<> builder(before);
@@ -645,31 +661,10 @@ ParallelRegion::InjectPrintF
        /*Name=*/"printf", M); 
     printfFunc->setCallingConv(CallingConv::C);
 
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-    AttrListPtr func_printf_PAL;
-#else
     AttributeSet func_printf_PAL;
-#endif
     {
-#ifdef LLVM_3_1
-      SmallVector<AttributeWithIndex, 4> Attrs;
-      AttributeWithIndex PAWI;
-      PAWI.Index = 1U; 
-      PAWI.Attrs = Attribute::NoCapture;
-      Attrs.push_back(PAWI);
-      PAWI.Index = 4294967295U; 
-      PAWI.Attrs = Attribute::NoUnwind;
-      Attrs.push_back(PAWI);
-      func_printf_PAL = AttrListPtr::get(Attrs.begin(), Attrs.end());
-#elif defined LLVM_3_2
-      SmallVector<AttributeWithIndex, 4> Attrs;
-      Attrs.push_back(AttributeWithIndex::get(M->getContext(), 1U, Attributes::NoCapture));
-      Attrs.push_back(AttributeWithIndex::get(M->getContext(), 4294967295U, Attributes::NoUnwind));
-      func_printf_PAL = AttrListPtr::get(M->getContext(), Attrs);
-#else
       func_printf_PAL.addAttribute( M->getContext(), 1U, Attribute::NoCapture);
       func_printf_PAL.addAttribute( M->getContext(), 4294967295U, Attribute::NoUnwind);
-#endif
     }
     printfFunc->setAttributes(func_printf_PAL);
   }
@@ -680,9 +675,15 @@ ParallelRegion::InjectPrintF
   const_ptr_8_indices.push_back(const_int64_9);
   const_ptr_8_indices.push_back(const_int64_9);
   assert (isa<Constant>(stringArg));
-  Constant* const_ptr_8 = 
+  #ifdef LLVM_OLDER_THAN_3_7
+  Constant* const_ptr_8 =
     ConstantExpr::getGetElementPtr
     (cast<Constant>(stringArg), const_ptr_8_indices);
+  #else
+  Constant* const_ptr_8 =
+    ConstantExpr::getGetElementPtr
+    (PointerType::getUnqual(Type::getInt8Ty(getGlobalContext())), cast<Constant>(stringArg), const_ptr_8_indices);
+  #endif
 
   std::vector<Value*> args;
   args.push_back(const_ptr_8);
@@ -754,7 +755,7 @@ ParallelRegion::InjectVariablePrintouts()
       for (llvm::BasicBlock::iterator instr = bb->begin();
            instr != bb->end(); ++instr) 
         {
-          llvm::Instruction *instruction = instr;
+          llvm::Instruction *instruction = &*instr;
           if (isa<PointerType>(instruction->getType()) ||
               !instruction->hasName()) continue;
           std::string name = instruction->getName().str();
@@ -803,7 +804,7 @@ ParallelRegion::LocalizeIDLoads()
       for (llvm::BasicBlock::iterator instrI = bb->begin();
            instrI != bb->end(); ++instrI) 
         {
-	  llvm::Instruction *instr = instrI;
+          llvm::Instruction *instr = &*instrI;
 	  if (instr == LocalIDXLoadInstr ||
 	      instr == LocalIDYLoadInstr ||
 	      instr == LocalIDZLoadInstr) continue;
