@@ -2,6 +2,7 @@
 // and parallelized kernel for an OpenCL workgroup.
 // 
 // Copyright (c) 2011 Universidad Rey Juan Carlos
+//               2012-2015 Pekka Jääskeläinen
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,33 +22,18 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "Barrier.h"
-#include "Workgroup.h"
+#include <cstdio>
+#include <map>
+#include <iostream>
+
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include "pocl.h"
 
-#include "CanonicalizeBarriers.h"
-#include "BarrierTailReplication.h"
-#include "WorkitemReplication.h"
 #include "llvm/Analysis/ConstantFolding.h"
 #include "llvm/Pass.h"
 #include "llvm/Support/CommandLine.h"
-#include "config.h"
-#ifdef LLVM_3_1
-#include "llvm/Support/IRBuilder.h"
-#include "llvm/Support/TypeBuilder.h"
-#include "llvm/BasicBlock.h"
-#include "llvm/Constants.h"
-#include "llvm/DerivedTypes.h"
-#include "llvm/InstrTypes.h"
-#include "llvm/Module.h"
-#elif defined LLVM_3_2
-#include "llvm/IRBuilder.h"
-#include "llvm/TypeBuilder.h"
-#include "llvm/BasicBlock.h"
-#include "llvm/Constants.h"
-#include "llvm/DerivedTypes.h"
-#include "llvm/InstrTypes.h"
-#include "llvm/Module.h"
-#else
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/TypeBuilder.h"
 #include "llvm/IR/BasicBlock.h"
@@ -55,18 +41,25 @@
 #include "llvm/IR/DerivedTypes.h"
 #include "llvm/IR/InstrTypes.h"
 #include "llvm/IR/Module.h"
-#endif
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "llvm/Transforms/Utils/Local.h"
-#include <cstdio>
-#include <map>
-#include <iostream>
 
-#include "pocl.h"
+#include "CanonicalizeBarriers.h"
+#include "BarrierTailReplication.h"
+#include "WorkitemReplication.h"
+#include "Barrier.h"
+#include "Workgroup.h"
+
+
+#if _MSC_VER
+#  include "vccompat.hpp"
+#endif
 
 #define STRING_LENGTH 32
 
+POP_COMPILER_DIAGS
+
 using namespace std;
 using namespace llvm;
 using namespace pocl;
@@ -77,7 +70,6 @@ static void createWorkgroup(Module &M, Function *F);
 static void createWorkgroupFast(Module &M, Function *F);
 
 // extern cl::opt<string> Header;
-// extern cl::list<int> LocalSize;
 
 /* The kernel to process in this kernel compiler launch. */
 cl::opt<string>
@@ -154,26 +146,25 @@ bool
 Workgroup::runOnModule(Module &M)
 {
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  if (M.getPointerSize() == llvm::Module::Pointer64)
+#ifdef LLVM_OLDER_THAN_3_7
+  if (M.getDataLayout()->getPointerSize(0) == 8)
     {
       TypeBuilder<PoclContext, true>::setSizeTWidth(64);
     }
-  else if (M.getPointerSize() == llvm::Module::Pointer32) 
+  else if (M.getDataLayout()->getPointerSize(0) == 4)
     {
       TypeBuilder<PoclContext, true>::setSizeTWidth(32);
     }
   else 
     {
       assert (false && "Target has an unsupported pointer width.");
-    }  
+    }
 #else
-// FIXME 0 here is the address space - this breaks (?) if _local_size_x is not stored in AS0
-  if (M.getDataLayout()->getPointerSize(0) == 8)
+  if (M.getDataLayout().getPointerSize(0) == 8)
     {
       TypeBuilder<PoclContext, true>::setSizeTWidth(64);
     }
-  else if (M.getDataLayout()->getPointerSize(0) == 4)
+  else if (M.getDataLayout().getPointerSize(0) == 4)
     {
       TypeBuilder<PoclContext, true>::setSizeTWidth(32);
     }
@@ -190,13 +181,9 @@ Workgroup::runOnModule(Module &M)
 
   for (Module::iterator i = M.begin(), e = M.end(); i != e; ++i) {
     if (!isKernelToProcess(*i)) continue;
-    Function *L = createLauncher(M, i);
+    Function *L = createLauncher(M, &*i);
       
-#if defined LLVM_3_2
-    L->addFnAttr(Attributes::NoInline);
-#else
     L->addFnAttr(Attribute::NoInline);
-#endif
 
     privatizeContext(M, L);
 
@@ -233,13 +220,13 @@ createLauncher(Module &M, Function *F)
 
   Function *L = Function::Create(ft,
 				 Function::ExternalLinkage,
-				 "_" + funcName,
+				 "_pocl_launcher_" + funcName,
 				 &M);
 
   SmallVector<Value *, 8> arguments;
   Function::arg_iterator ai = L->arg_begin();
   for (unsigned i = 0, e = F->getArgumentList().size(); i != e; ++i)  {
-    arguments.push_back(ai);
+    arguments.push_back(&*ai);
     ++ai;
   }  
 
@@ -253,8 +240,13 @@ createLauncher(Module &M, Function *F)
 
   IRBuilder<> builder(BasicBlock::Create(M.getContext(), "", L));
 
+#ifdef LLVM_OLDER_THAN_3_7
   ptr = builder.CreateStructGEP(ai,
 				TypeBuilder<PoclContext, true>::WORK_DIM);
+#else
+  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
+                                TypeBuilder<PoclContext, true>::WORK_DIM);
+#endif
   gv = M.getGlobalVariable("_work_dim");
   if (gv != NULL) {
     v = builder.CreateLoad(builder.CreateConstGEP1_32(ptr, 0));
@@ -263,16 +255,20 @@ createLauncher(Module &M, Function *F)
 
 
   int size_t_width = 32;
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  if (M.getPointerSize() == llvm::Module::Pointer64)
-#else
-  //FIXME 0 here is the address space: this breaks (?) if _local_size_x is not stored in AS0
+#ifdef LLVM_OLDER_THAN_3_7
   if (M.getDataLayout()->getPointerSize(0) == 8)
+#else
+  if (M.getDataLayout().getPointerSize(0) == 8)
 #endif
     size_t_width = 64;
 
+#ifdef LLVM_OLDER_THAN_3_7
   ptr = builder.CreateStructGEP(ai,
 				TypeBuilder<PoclContext, true>::GROUP_ID);
+#else
+  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
+                                TypeBuilder<PoclContext, true>::GROUP_ID);
+#endif
   for (int i = 0; i < 3; ++i) {
     snprintf(s, STRING_LENGTH, "_group_id_%c", 'x' + i);
     gv = M.getGlobalVariable(s);
@@ -283,14 +279,23 @@ createLauncher(Module &M, Function *F)
         }
       else
         {
+#ifdef LLVM_OLDER_THAN_3_7
           v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
+#else
+          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
+#endif
         }
       builder.CreateStore(v, gv);
     }
   }
 
+#ifdef LLVM_OLDER_THAN_3_7
   ptr = builder.CreateStructGEP(ai,
 				TypeBuilder<PoclContext, true>::NUM_GROUPS);
+#else
+  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
+                                TypeBuilder<PoclContext, true>::NUM_GROUPS);
+#endif
   for (int i = 0; i < 3; ++i) {
     snprintf(s, STRING_LENGTH, "_num_groups_%c", 'x' + i);
     gv = M.getGlobalVariable(s);
@@ -301,14 +306,23 @@ createLauncher(Module &M, Function *F)
         }
       else 
         {
+#ifdef LLVM_OLDER_THAN_3_7
           v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
+#else
+          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
+#endif
         }
       builder.CreateStore(v, gv);
     }
   }
 
+#ifdef LLVM_OLDER_THAN_3_7
   ptr = builder.CreateStructGEP(ai,
 				TypeBuilder<PoclContext, true>::GLOBAL_OFFSET);
+#else
+  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
+                                TypeBuilder<PoclContext, true>::GLOBAL_OFFSET);
+#endif
   for (int i = 0; i < 3; ++i) {
     snprintf(s, STRING_LENGTH, "_global_offset_%c", 'x' + i);
     gv = M.getGlobalVariable(s);
@@ -319,7 +333,11 @@ createLauncher(Module &M, Function *F)
         }
       else
         {
+#ifdef LLVM_OLDER_THAN_3_7
           v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
+#else
+          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
+#endif
         }
       builder.CreateStore(v, gv);
     }
@@ -497,7 +515,7 @@ createWorkgroup(Module &M, Function *F)
        ii != ee; ++ii) {
     Type *t = ii->getType();
 
-    Value *gep = builder.CreateGEP(ai,
+    Value *gep = builder.CreateGEP(&*ai,
             ConstantInt::get(IntegerType::get(M.getContext(), 32), i));
     Value *pointer = builder.CreateLoad(gep);
 
@@ -505,17 +523,9 @@ createWorkgroup(Module &M, Function *F)
      * as is to the function, no need to load form it first. */
     Value *value;
     if (ii->hasByValAttr()) {
-#if defined(LLVM_3_2) || defined(LLVM_3_3)
-        value = builder.CreateBitCast(pointer, t);
-#else
         value = builder.CreatePointerCast(pointer, t);
-#endif
     } else {
-#if defined(LLVM_3_2) || defined(LLVM_3_3)
-        value = builder.CreateBitCast(pointer, t->getPointerTo());
-#else
         value = builder.CreatePointerCast(pointer, t->getPointerTo());
-#endif
         value = builder.CreateLoad(value);
     }
 
@@ -523,7 +533,7 @@ createWorkgroup(Module &M, Function *F)
     ++i;
   }
 
-  arguments.back() = ++ai;
+  arguments.back() = &*(++ai);
   
   builder.CreateCall(F, ArrayRef<Value*>(arguments));
   builder.CreateRetVoid();
@@ -557,6 +567,8 @@ createWorkgroupFast(Module &M, Function *F)
     dyn_cast<Function>(M.getOrInsertFunction(funcName + "_workgroup_fast", ft));
   assert(workgroup != NULL);
 
+  workgroup->addFnAttr(Attribute::NoInline);
+
   builder.SetInsertPoint(BasicBlock::Create(M.getContext(), "", workgroup));
 
   Function::arg_iterator ai = workgroup->arg_begin();
@@ -566,18 +578,14 @@ createWorkgroupFast(Module &M, Function *F)
   for (Function::const_arg_iterator ii = F->arg_begin(), ee = F->arg_end();
        ii != ee; ++i, ++ii) {
     Type *t = ii->getType();
-    Value *gep = builder.CreateGEP(ai, 
+    Value *gep = builder.CreateGEP(&*ai,
             ConstantInt::get(IntegerType::get(M.getContext(), 32), i));
     Value *pointer = builder.CreateLoad(gep);
      
     if (t->isPointerTy()) {
       if (!ii->hasByValAttr()) {
         /* Assume the pointer is directly in the arg array. */
-#if defined(LLVM_3_2) || defined(LLVM_3_3)
-        arguments.push_back(builder.CreateBitCast(pointer, t));
-#else
         arguments.push_back(builder.CreatePointerCast(pointer, t));
-#endif
         continue;
       }
 
@@ -588,21 +596,22 @@ createWorkgroupFast(Module &M, Function *F)
 
     /* If it's a pass by value pointer argument, we just pass the pointer
      * as is to the function, no need to load from it first. */
-#if defined(LLVM_3_2) || defined(LLVM_3_3)
-    Value *value = builder.CreateBitCast
-      (pointer, t->getPointerTo(POCL_ADDRESS_SPACE_GLOBAL));
-#else
-    Value *value = builder.CreatePointerCast
-      (pointer, t->getPointerTo(POCL_ADDRESS_SPACE_GLOBAL));
-#endif
+    Value *value;
+
+    if (!ii->hasByValAttr() || ((PointerType*)t)->getAddressSpace() == 1)
+      value = builder.CreatePointerCast
+        (pointer, t->getPointerTo(POCL_ADDRESS_SPACE_GLOBAL));
+    else
+      value = builder.CreatePointerCast(pointer, t->getPointerTo());
+
     if (!ii->hasByValAttr()) {
-        value = builder.CreateLoad(value);
+      value = builder.CreateLoad(value);
     }
     
     arguments.push_back(value);
   }
 
-  arguments.back() = ++ai;
+  arguments.back() = &*(++ai);
   
   builder.CreateCall(F, ArrayRef<Value*>(arguments));
   builder.CreateRetVoid();
@@ -631,10 +640,39 @@ Workgroup::isKernelToProcess(const Function &F)
   for (unsigned i = 0, e = kernels->getNumOperands(); i != e; ++i) {
     if (kernels->getOperand(i)->getOperand(0) == NULL)
       continue; // globaldce might have removed uncalled kernels
-    Function *k = cast<Function>(kernels->getOperand(i)->getOperand(0));
+    Function *k = 
+      cast<Function>(
+        dyn_cast<ValueAsMetadata>(kernels->getOperand(i)->getOperand(0))
+          ->getValue());
     if (&F == k)
       return true;
   }
 
   return false;
 }
+
+/**
+ * Returns true in case the given function is a kernel 
+ * with work-group barriers inside it.
+ */
+bool
+Workgroup::hasWorkgroupBarriers(const Function &F)
+{
+  for (llvm::Function::const_iterator i = F.begin(), e = F.end();
+       i != e; ++i) {
+    const llvm::BasicBlock* bb = &*i;
+    if (Barrier::hasBarrier(bb)) {
+
+      // Ignore the implicit entry and exit barriers.
+      if (Barrier::hasOnlyBarrier(bb) && bb == &F.getEntryBlock())
+        continue;
+
+      if (Barrier::hasOnlyBarrier(bb) && 
+          bb->getTerminator()->getNumSuccessors() == 0) 
+        continue;
+
+      return true;
+    }
+  }
+  return false;
+}
