@@ -2,7 +2,7 @@
 // in a work group.
 // 
 // Copyright (c) 2011-2012 Carlos Sánchez de La Lama / URJC and
-//               2012-2014 Pekka Jääskeläinen / TUT
+//               2012-2015 Pekka Jääskeläinen / TUT
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -22,38 +22,40 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "config.h"
 #include <sstream>
 #include <iostream>
 
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-#include "llvm/Metadata.h"
-#include "llvm/Constants.h"
-#include "llvm/Module.h"
-#include "llvm/Instructions.h"
-#include "llvm/ValueSymbolTable.h"
-#else
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include "pocl.h"
+
 #include "llvm/IR/Metadata.h"
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/ValueSymbolTable.h"
-#endif
 #include "llvm/Support/CommandLine.h"
+
 #include "WorkitemHandler.h"
 #include "Kernel.h"
 #include "DebugHelpers.h"
 
+POP_COMPILER_DIAGS
+
 //#define DEBUG_REFERENCE_FIXING
 
 namespace pocl {
 
 using namespace llvm;
 
-llvm::cl::list<int>
-LocalSize("local-size",
-          llvm::cl::desc("Local size (x y z)"),
-          llvm::cl::multi_val(3));
+/* This is used to communicate the work-group dimensions of the currently
+   compiled kernel command to the workitem loop. 
+
+   TODO: Something cleaner than a global value. */
+size_t WGLocalSizeX = 1;
+size_t WGLocalSizeY = 1;
+size_t WGLocalSizeZ = 1;
 
 cl::opt<bool>
 AddWIMetadata("add-wi-metadata", cl::init(false), cl::Hidden,
@@ -64,7 +66,7 @@ WorkitemHandler::WorkitemHandler(char& ID) : FunctionPass(ID) {
 }
 
 bool
-WorkitemHandler::runOnFunction(Function &F) {
+WorkitemHandler::runOnFunction(Function &) {
   return false;
 }
 
@@ -73,38 +75,55 @@ WorkitemHandler::Initialize(Kernel *K) {
 
   llvm::Module *M = K->getParent();
   
-  LocalSizeX = LocalSize[0];
-  LocalSizeY = LocalSize[1];
-  LocalSizeZ = LocalSize[2];
-  
-  llvm::NamedMDNode *size_info = M->getNamedMetadata("opencl.kernel_wg_size_info");
+  // Check that the dynamically set local size and a possible
+  // required WG size match.
+  size_t LocalSizeX = WGLocalSizeX, LocalSizeY = WGLocalSizeY, 
+      LocalSizeZ = WGLocalSizeZ;
+ 
+  llvm::NamedMDNode *size_info = 
+    M->getNamedMetadata("opencl.kernel_wg_size_info");
   if (size_info) {
     for (unsigned i = 0, e = size_info->getNumOperands(); i != e; ++i) {
       llvm::MDNode *KernelSizeInfo = size_info->getOperand(i);
-      if (KernelSizeInfo->getOperand(0) == K) {
-        LocalSizeX = (llvm::cast<ConstantInt>(
-          KernelSizeInfo->getOperand(1)))->getLimitedValue();
-        LocalSizeY = (llvm::cast<ConstantInt>(
-          KernelSizeInfo->getOperand(2)))->getLimitedValue();
-        LocalSizeZ = (llvm::cast<ConstantInt>(
-          KernelSizeInfo->getOperand(3)))->getLimitedValue();
-      }
+      if (dyn_cast<ValueAsMetadata>(
+        KernelSizeInfo->getOperand(0).get())->getValue() != K) 
+        continue;
+
+      LocalSizeX = (llvm::cast<ConstantInt>(
+                     llvm::dyn_cast<ConstantAsMetadata>(
+                       KernelSizeInfo->getOperand(1))->getValue()))->getLimitedValue();
+      LocalSizeY = (llvm::cast<ConstantInt>(
+                     llvm::dyn_cast<ConstantAsMetadata>(
+                       KernelSizeInfo->getOperand(2))->getValue()))->getLimitedValue();
+      LocalSizeZ = (llvm::cast<ConstantInt>(
+                     llvm::dyn_cast<ConstantAsMetadata>(
+                       KernelSizeInfo->getOperand(3))->getValue()))->getLimitedValue();
+      break;
     }
   }
 
+  // This funny looking check is to silence a compiler warning that we 
+  // do not ignore the LocalSize* variables. Even in a NDEBUG build.
+  // TODO: how to handle the case in a NDEBUG build when they don't match?
+  if( !(LocalSizeX == WGLocalSizeX && LocalSizeY == WGLocalSizeY && 
+          LocalSizeZ == WGLocalSizeZ) ){
+     assert(false && "Local sizes don't match");
+     return;
+  }
+
   llvm::Type *localIdType; 
   size_t_width = 0;
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  if (M->getPointerSize() == llvm::Module::Pointer64)
+#ifdef LLVM_OLDER_THAN_3_7
+  if (M->getDataLayout()->getPointerSize(0) == 8)
     size_t_width = 64;
-  else if (M->getPointerSize() == llvm::Module::Pointer32)
+  else if (M->getDataLayout()->getPointerSize(0) == 4)
     size_t_width = 32;
   else
     assert (false && "Only 32 and 64 bit size_t widths supported.");
 #else
-  if (M->getDataLayout()->getPointerSize(0) == 8)
+  if (M->getDataLayout().getPointerSize(0) == 8)
     size_t_width = 64;
-  else if (M->getDataLayout()->getPointerSize(0) == 4)
+  else if (M->getDataLayout().getPointerSize(0) == 4)
     size_t_width = 32;
   else
     assert (false && "Only 32 and 64 bit size_t widths supported.");
@@ -118,16 +137,10 @@ WorkitemHandler::Initialize(Kernel *K) {
 }
 
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-bool
-WorkitemHandler::dominatesUse
-(llvm::DominatorTree *DT, Instruction &I, unsigned i) {
-#else
 bool
 WorkitemHandler::dominatesUse
 (llvm::DominatorTreeWrapperPass *DTP, Instruction &I, unsigned i) {
   DominatorTree *DT = &DTP->getDomTree();
-#endif
   Instruction *Op = cast<Instruction>(I.getOperand(i));
   BasicBlock *OpBlock = Op->getParent();
   PHINode *PN = dyn_cast<PHINode>(&I);
@@ -184,22 +197,16 @@ WorkitemHandler::dominatesUse
    the old one. This should ensure the reachability without 
    the costly dominance analysis.
 */
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-bool
-WorkitemHandler::fixUndominatedVariableUses(llvm::DominatorTree *DT, 
-                                            llvm::Function &F) 
-#else
 bool
 WorkitemHandler::fixUndominatedVariableUses(llvm::DominatorTreeWrapperPass *DT,
                                             llvm::Function &F)
-#endif
 {
   bool changed = false;
   DT->runOnFunction(F);
 
   for (Function::iterator i = F.begin(), e = F.end(); i != e; ++i) 
     {
-      llvm::BasicBlock *bb = i;
+      llvm::BasicBlock *bb = &*i;
       for (llvm::BasicBlock::iterator ins = bb->begin(), inse = bb->end();
            ins != inse; ++ins)
         {
