@@ -24,20 +24,18 @@
 #ifndef _POCL_WORKITEM_HANDLER_H
 #define _POCL_WORKITEM_HANDLER_H
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
 #include "config.h"
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-#include "llvm/Function.h"
-#else
+
 #include "llvm/IR/Function.h"
-#endif
-
-#if not (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
 #include "llvm/IR/Dominators.h"
-#endif
-
 #include "llvm/Pass.h"
 #include "llvm/Support/CommandLine.h"
 
+POP_COMPILER_DIAGS
+
 namespace llvm {
   class DominatorTree;
 }
@@ -59,15 +57,8 @@ namespace pocl {
   protected:
     
     void movePhiNodes(llvm::BasicBlock* src, llvm::BasicBlock* dst);
-    #if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-    bool fixUndominatedVariableUses(llvm::DominatorTree *DT, llvm::Function &F);
-    bool dominatesUse(llvm::DominatorTree *DT, llvm::Instruction &I, unsigned i);
-    #else
     bool fixUndominatedVariableUses(llvm::DominatorTreeWrapperPass *DT, llvm::Function &F);
     bool dominatesUse(llvm::DominatorTreeWrapperPass *DT, llvm::Instruction &I, unsigned i);
-    #endif
-
-    int LocalSizeX, LocalSizeY, LocalSizeZ;
 
     unsigned size_t_width;
 
@@ -78,6 +69,10 @@ namespace pocl {
 
   extern llvm::cl::opt<bool> AddWIMetadata;
   extern llvm::cl::opt<int> LockStepSIMDWidth;
+
+  extern size_t WGLocalSizeX;
+  extern size_t WGLocalSizeY;
+  extern size_t WGLocalSizeZ;
 }
 
 #endif
