@@ -23,46 +23,31 @@
 
 #define DEBUG_TYPE "workitem-loops"
 
-#include "WorkitemLoops.h"
-#include "Workgroup.h"
-#include "Barrier.h"
-#include "Kernel.h"
-#include "config.h"
+#include <iostream>
+#include <map>
+#include <sstream>
+#include <vector>
+
+#include "pocl.h"
+
 #include "llvm/ADT/Statistic.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Support/CommandLine.h"
-#ifdef LLVM_3_1
-#include "llvm/Support/IRBuilder.h"
-#include "llvm/Support/TypeBuilder.h"
-#include "llvm/Target/TargetData.h"
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#include "llvm/ValueSymbolTable.h"
-#elif defined LLVM_3_2
-#include "llvm/IRBuilder.h"
-#include "llvm/TypeBuilder.h"
-#include "llvm/DataLayout.h"
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#include "llvm/ValueSymbolTable.h"
-#else
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/TypeBuilder.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/ValueSymbolTable.h"
-#endif
 #include "llvm/Analysis/PostDominators.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 
+#include "WorkitemLoops.h"
+#include "Workgroup.h"
+#include "Barrier.h"
+#include "Kernel.h"
 #include "WorkitemHandlerChooser.h"
 
-#include <iostream>
-#include <map>
-#include <sstream>
-#include <vector>
-
 //#define DUMP_CFGS
 
 #include "DebugHelpers.h"
@@ -89,17 +74,12 @@ WorkitemLoops::getAnalysisUsage(AnalysisUsage &AU) const
 {
 
   AU.addRequired<PostDominatorTree>();
+#ifdef LLVM_OLDER_THAN_3_7
   AU.addRequired<LoopInfo>();
-#ifdef LLVM_3_1
-  AU.addRequired<TargetData>();
-#endif
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  AU.addRequired<DominatorTree>();
-  AU.addRequired<DataLayout>();
 #else
-  AU.addRequired<DominatorTreeWrapperPass>();
-  AU.addRequired<DataLayoutPass>();
+  AU.addRequired<LoopInfoWrapperPass>();
 #endif
+  AU.addRequired<DominatorTreeWrapperPass>();
 
   AU.addRequired<VariableUniformityAnalysis>();
   AU.addPreserved<pocl::VariableUniformityAnalysis>();
@@ -119,22 +99,17 @@ WorkitemLoops::runOnFunction(Function &F)
       pocl::WorkitemHandlerChooser::POCL_WIH_LOOPS)
     return false;
 
-  #if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  DT = &getAnalysis<DominatorTree>();
-  #else
   DTP = &getAnalysis<DominatorTreeWrapperPass>();
   DT = &DTP->getDomTree();
-  #endif
+#ifdef LLVM_OLDER_THAN_3_7
   LI = &getAnalysis<LoopInfo>();
+#else
+  LI = &getAnalysis<LoopInfoWrapperPass>();
+#endif
   PDT = &getAnalysis<PostDominatorTree>();
 
   tempInstructionIndex = 0;
 
-#if 0
-  std::cerr << "### original:" << std::endl;
-  chopBBs(F, *this);
-  F.viewCFG();
-#endif
 //  F.viewCFGOnly();
 
   bool changed = ProcessFunction(F);
@@ -149,11 +124,7 @@ WorkitemLoops::runOnFunction(Function &F)
   F.viewCFG();
 #endif
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  changed |= fixUndominatedVariableUses(DT, F);
-#else
   changed |= fixUndominatedVariableUses(DTP, F);
-#endif
 
 #if 0
   /* Split large BBs so we can print the Dot without it crashing. */
@@ -241,11 +212,7 @@ WorkitemLoops::CreateLoopAround
     BasicBlock::Create(C, "pregion_for_cond", F, exitBB);
 
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  DT->runOnFunction(*F);
-#else
   DTP->runOnFunction(*F);
-#endif
 
   //  F->viewCFG();
   /* Fix the old edges jumping to the region to jump to the basic block
@@ -312,7 +279,12 @@ WorkitemLoops::CreateLoopAround
 
   /* This creation of the identifier metadata is copied from
      LLVM's MDBuilder::createAnonymousTBAARoot(). */
-  MDNode *Dummy = MDNode::getTemporary(C, ArrayRef<Value*>());
+#ifdef LLVM_OLDER_THAN_3_7
+  MDNode *Dummy = MDNode::getTemporary(C, ArrayRef<Metadata*>());
+#else
+  MDNode *Dummy = MDNode::getTemporary(C, ArrayRef<Metadata*>()).release();
+#endif
+
   MDNode *Root = MDNode::get(C, Dummy);
   // At this point we have
   //   !0 = metadata !{}            <- dummy
@@ -323,13 +295,10 @@ WorkitemLoops::CreateLoopAround
   // We now have
   //   !1 = metadata !{metadata !1} <- self-referential root
 
-#ifdef LLVM_3_3
-  loopBranch->setMetadata("llvm.loop.parallel", Root);
-#else
   loopBranch->setMetadata("llvm.loop", Root);
-#endif
   region.AddParallelLoopMetadata(Root);
 
+
   builder.SetInsertPoint(loopEndBB);
   builder.CreateBr(oldExit);
 
@@ -355,17 +324,20 @@ WorkitemLoops::ProcessFunction(Function &F)
 {
   Kernel *K = cast<Kernel> (&F);
   Initialize(K);
-  unsigned workItemCount = LocalSizeX*LocalSizeY*LocalSizeZ;
+  unsigned workItemCount = WGLocalSizeX*WGLocalSizeY*WGLocalSizeZ;
 
   if (workItemCount == 1)
     {
-      K->addLocalSizeInitCode(LocalSizeX, LocalSizeY, LocalSizeZ);
+      K->addLocalSizeInitCode(WGLocalSizeX, WGLocalSizeY, WGLocalSizeZ);
       ParallelRegion::insertLocalIdInit(&F.getEntryBlock(), 0, 0, 0);
       return true;
     }
 
-  original_parallel_regions =
-    K->getParallelRegions(LI);
+#ifdef LLVM_OLDER_THAN_3_7
+  original_parallel_regions = K->getParallelRegions(LI);
+#else
+  original_parallel_regions = K->getParallelRegions(&LI->getLoopInfo());
+#endif
 
 #ifdef DUMP_CFGS
   F.dump();
@@ -373,7 +345,7 @@ WorkitemLoops::ProcessFunction(Function &F)
           original_parallel_regions);
 #endif
 
-  IRBuilder<> builder(F.getEntryBlock().getFirstInsertionPt());
+  IRBuilder<> builder(&*(F.getEntryBlock().getFirstInsertionPt()));
   localIdXFirstVar = 
     builder.CreateAlloca
     (IntegerType::get(F.getContext(), size_t_width), 0, ".pocl.local_id_x_init");
@@ -483,7 +455,7 @@ WorkitemLoops::ProcessFunction(Function &F)
             preds.push_back(bb);
           }
 
-        int unrollCount;
+        unsigned unrollCount;
         if (getenv("POCL_WILOOPS_MAX_UNROLL_COUNT") != NULL)
             unrollCount = atoi(getenv("POCL_WILOOPS_MAX_UNROLL_COUNT"));
         else
@@ -491,8 +463,8 @@ WorkitemLoops::ProcessFunction(Function &F)
         /* Find a two's exponent unroll count, if available. */
         while (unrollCount >= 1)
           {
-            if (LocalSizeX % unrollCount == 0 &&
-                unrollCount <= LocalSizeX)
+            if (WGLocalSizeX % unrollCount == 0 &&
+                unrollCount <= WGLocalSizeX)
               {
                 break;
               }
@@ -509,7 +481,7 @@ WorkitemLoops::ProcessFunction(Function &F)
             if (AddWIMetadata)
                 original->AddIDMetadata(F.getContext(), 0);
 
-            for (int c = 1; c < unrollCount; ++c) 
+            for (unsigned c = 1; c < unrollCount; ++c) 
             {
                 ParallelRegion *unrolled = 
                     original->replicate(reference_map, ".unrolled_wi");
@@ -526,14 +498,20 @@ WorkitemLoops::ProcessFunction(Function &F)
         }
       }
 
-    if (LocalSizeX > 1)
-      l = CreateLoopAround(*original, l.first, l.second, peelFirst, localIdX, LocalSizeX, !unrolled);
+    if (WGLocalSizeX > 1)
+      l = CreateLoopAround(
+        *original, l.first, l.second, peelFirst, 
+        localIdX, WGLocalSizeX, !unrolled);
 
-    if (LocalSizeY > 1)
-      l = CreateLoopAround(*original, l.first, l.second, false, localIdY, LocalSizeY);
+    if (WGLocalSizeY > 1)
+      l = CreateLoopAround(
+        *original, l.first, l.second, 
+        false, localIdY, WGLocalSizeY);
 
-    if (LocalSizeZ > 1)
-      l = CreateLoopAround(*original, l.first, l.second, false, localIdZ, LocalSizeZ);
+    if (WGLocalSizeZ > 1)
+      l = CreateLoopAround(
+          *original, l.first, l.second, 
+          false, localIdZ, WGLocalSizeZ);
 
     /* Loop edges coming from another region mean B-loops which means 
        we have to fix the loop edge to jump to the beginning of the wi-loop 
@@ -564,13 +542,13 @@ WorkitemLoops::ProcessFunction(Function &F)
 
     if (!peeledRegion[pr]) continue;
     pr->insertPrologue(0, 0, 0);
-    builder.SetInsertPoint(pr->entryBB()->getFirstInsertionPt());
+    builder.SetInsertPoint(&*(pr->entryBB()->getFirstInsertionPt()));
     builder.CreateStore
       (ConstantInt::get(IntegerType::get(F.getContext(), size_t_width), 1), 
        localIdXFirstVar);       
   }
 
-  K->addLocalSizeInitCode(LocalSizeX, LocalSizeY, LocalSizeZ);
+  K->addLocalSizeInitCode(WGLocalSizeX, WGLocalSizeY, WGLocalSizeZ);
   ParallelRegion::insertLocalIdInit(&F.getEntryBlock(), 0, 0, 0);
 
 #if 0
@@ -604,7 +582,7 @@ WorkitemLoops::FixMultiRegionVariables(ParallelRegion *region)
       for (llvm::BasicBlock::iterator instr = bb->begin();
            instr != bb->end(); ++instr) 
         {
-          llvm::Instruction *instruction = instr;
+          llvm::Instruction *instruction = &*instr;
           instructionsInRegion.insert(instruction);
         }
     }
@@ -618,19 +596,15 @@ WorkitemLoops::FixMultiRegionVariables(ParallelRegion *region)
       for (llvm::BasicBlock::iterator instr = bb->begin();
            instr != bb->end(); ++instr) 
         {
-          llvm::Instruction *instruction = instr;
+          llvm::Instruction *instruction = &*instr;
 
-          if (ShouldNotBeContextSaved(instr)) continue;
+          if (ShouldNotBeContextSaved(&*instr)) continue;
 
           for (Instruction::use_iterator ui = instruction->use_begin(),
                  ue = instruction->use_end();
                ui != ue; ++ui) 
             {
-#if defined LLVM_3_2 || defined LLVM_3_3 || defined LLVM_3_4
-              llvm::Instruction *user = dyn_cast<Instruction>(*ui);
-#else
               llvm::Instruction *user = dyn_cast<Instruction>(ui->getUser());
-#endif
 
               if (user == NULL) continue;
               // If the instruction is used outside this region inside another
@@ -677,12 +651,15 @@ WorkitemLoops::AddContextSave
     }
 
   /* Save the produced variable to the array. */
-  BasicBlock::iterator definition = dyn_cast<Instruction>(instruction);
-
+#ifdef LLVM_OLDER_THAN_3_8
+  BasicBlock::iterator definition = (dyn_cast<Instruction>(instruction));
+#else
+  BasicBlock::iterator definition = (dyn_cast<Instruction>(instruction))->getIterator();
+#endif
   ++definition;
   while (isa<PHINode>(definition)) ++definition;
 
-  IRBuilder<> builder(definition); 
+  IRBuilder<> builder(&*definition);
   std::vector<llvm::Value *> gepArgs;
   gepArgs.push_back(ConstantInt::get(IntegerType::get(instruction->getContext(), size_t_width), 0));
 
@@ -782,7 +759,8 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
   if (contextArrays.find(varName) != contextArrays.end())
     return contextArrays[varName];
 
-  IRBuilder<> builder(instruction->getParent()->getParent()->getEntryBlock().getFirstInsertionPt());
+  BasicBlock &bb = instruction->getParent()->getParent()->getEntryBlock();
+  IRBuilder<> builder(&*(bb.getFirstInsertionPt()));
 
   llvm::Type *elementType;
   if (isa<AllocaInst>(instruction))
@@ -808,8 +786,8 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
     ArrayType::get(
         ArrayType::get(
             ArrayType::get(
-                elementType, LocalSizeX), 
-            LocalSizeY), LocalSizeZ);
+                elementType, WGLocalSizeX), 
+            WGLocalSizeY), WGLocalSizeZ);
 
   /* Allocate the context data array for the variable. */
   llvm::AllocaInst *alloca = 
@@ -868,11 +846,7 @@ WorkitemLoops::AddContextSaveRestore
          ue = instruction->use_end();
        ui != ue; ++ui) 
     {
-#if defined LLVM_3_2 || defined LLVM_3_3 || defined LLVM_3_4
-      llvm::Instruction *user = cast<Instruction>(*ui);
-#else
       llvm::Instruction *user = cast<Instruction>(ui->getUser());
-#endif
       if (user == NULL) continue;
       if (user == theStore) continue;
       uses.push_back(user);
