@@ -23,15 +23,15 @@
 #ifndef _POCL_WORKITEM_LOOPS_H
 #define _POCL_WORKITEM_LOOPS_H
 
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-#include "llvm/Analysis/Dominators.h"
-#endif
+#include <map>
+#include <vector>
+
+#include "pocl.h"
 
 #include "llvm/ADT/Twine.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Transforms/Utils/ValueMapper.h"
-#include <map>
-#include <vector>
+
 #include "WorkitemHandler.h"
 #include "ParallelRegion.h"
 
@@ -60,11 +60,13 @@ namespace pocl {
     typedef std::map<std::string, llvm::Instruction*> StrInstructionMap;
 
     llvm::DominatorTree *DT;
+#ifdef LLVM_OLDER_THAN_3_7
     llvm::LoopInfo *LI;
-    llvm::PostDominatorTree *PDT;
-#if not (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-    llvm::DominatorTreeWrapperPass *DTP;
+#else
+    llvm::LoopInfoWrapperPass *LI;
 #endif
+    llvm::PostDominatorTree *PDT;
+    llvm::DominatorTreeWrapperPass *DTP;
 
     ParallelRegion::ParallelRegionVector *original_parallel_regions;
 
