@@ -2,7 +2,7 @@
 // in a work group.
 // 
 // Copyright (c) 2011-2012 Carlos Sánchez de La Lama / URJC and
-//               2011-2014 Pekka Jääskeläinen / TUT
+//               2011-2015 Pekka Jääskeläinen / TUT
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,43 +24,34 @@
 
 #define DEBUG_TYPE "workitem"
 
-#include "WorkitemReplication.h"
-#include "Workgroup.h"
-#include "Barrier.h"
-#include "Kernel.h"
+#include <iostream>
+#include <map>
+#include <sstream>
+#include <vector>
+
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include "pocl.h"
+
 #include "llvm/ADT/Statistic.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Support/CommandLine.h"
-#include "config.h"
-#ifdef LLVM_3_1
-#include "llvm/Support/IRBuilder.h"
-#include "llvm/Target/TargetData.h"
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#include "llvm/ValueSymbolTable.h"
-#elif defined LLVM_3_2
-#include "llvm/IRBuilder.h"
-#include "llvm/DataLayout.h"
-#include "llvm/Instructions.h"
-#include "llvm/Module.h"
-#include "llvm/ValueSymbolTable.h"
-#else
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/ValueSymbolTable.h"
-#endif
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
+
+#include "WorkitemReplication.h"
+#include "Workgroup.h"
+#include "Barrier.h"
+#include "Kernel.h"
 #include "WorkitemHandlerChooser.h"
 #include "DebugHelpers.h"
 #include "VariableUniformityAnalysis.h"
 
-#include <iostream>
-#include <map>
-#include <sstream>
-#include <vector>
-
 //#define DEBUG_BB_MERGING
 //#define DUMP_RESULT_CFG
 //#define DEBUG_PR_REPLICATION
@@ -69,6 +60,8 @@
 #include "llvm/Analysis/CFGPrinter.h"
 #endif
 
+POP_COMPILER_DIAGS
+
 using namespace llvm;
 using namespace pocl;
 
@@ -85,18 +78,14 @@ char WorkitemReplication::ID = 0;
 void
 WorkitemReplication::getAnalysisUsage(AnalysisUsage &AU) const
 {
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  AU.addRequired<DominatorTree>();
-#else
   AU.addRequired<DominatorTreeWrapperPass>();
-#endif
 
+#ifdef LLVM_OLDER_THAN_3_7
   AU.addRequired<LoopInfo>();
-#ifdef LLVM_3_1
-  AU.addRequired<TargetData>();
-#elif (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  AU.addRequired<DataLayout>();
 #else
+  AU.addRequired<LoopInfoWrapperPass>();
+#endif
+#ifdef LLVM_OLDER_THAN_3_7
   AU.addRequired<DataLayoutPass>();
 #endif
   AU.addRequired<pocl::WorkitemHandlerChooser>();
@@ -113,14 +102,14 @@ WorkitemReplication::runOnFunction(Function &F)
       pocl::WorkitemHandlerChooser::POCL_WIH_FULL_REPLICATION)
     return false;
 
-  #if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  DT = &getAnalysis<DominatorTree>();
-  #else
   DTP = &getAnalysis<DominatorTreeWrapperPass>();
   DT = &DTP->getDomTree();
-  #endif
 
+#ifdef LLVM_OLDER_THAN_3_7
   LI = &getAnalysis<LoopInfo>();
+#else
+  LI = &getAnalysis<LoopInfoWrapperPass>();
+#endif
 
   bool changed = ProcessFunction(F);
 #ifdef DUMP_RESULT_CFG
@@ -128,11 +117,7 @@ WorkitemReplication::runOnFunction(Function &F)
   cfgPrinter->runOnFunction(F);
 #endif
 
-  #if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  changed |= fixUndominatedVariableUses(DT, F);
-  #else
   changed |= fixUndominatedVariableUses(DTP, F);
-  #endif
   return changed;
 }
 
@@ -148,16 +133,23 @@ WorkitemReplication::ProcessFunction(Function &F)
 
   // Allocate space for workitem reference maps. Workitem 0 does
   // not need it.
-  unsigned workitem_count = LocalSizeZ * LocalSizeY * LocalSizeX;
+  unsigned workitem_count = WGLocalSizeZ * WGLocalSizeY * WGLocalSizeX;
+
+  assert (workitem_count > 0);
 
   BasicBlockVector original_bbs;
   for (Function::iterator i = F.begin(), e = F.end(); i != e; ++i) {
-      if (!Barrier::hasBarrier(i))
-        original_bbs.push_back(i);
+      if (!Barrier::hasBarrier(&*i))
+        original_bbs.push_back(&*i);
   }
 
+#ifdef LLVM_OLDER_THAN_3_7
   ParallelRegion::ParallelRegionVector* original_parallel_regions =
     K->getParallelRegions(LI);
+#else
+  ParallelRegion::ParallelRegionVector* original_parallel_regions =
+    K->getParallelRegions(&LI->getLoopInfo());
+#endif
 
   std::vector<ParallelRegion::ParallelRegionVector> parallel_regions(
       workitem_count);
@@ -180,13 +172,7 @@ WorkitemReplication::ProcessFunction(Function &F)
 #endif
   
   // Measure the required context (variables alive in more than one region).
-#ifdef LLVM_3_1
-  TargetData &TD = getAnalysis<TargetData>();
-#elif (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-  DataLayout &TD = getAnalysis<DataLayout>();
-#else
-  const DataLayout &TD = getAnalysis<DataLayoutPass>().getDataLayout();
-#endif
+  const DataLayout &TD = F.getParent()->getDataLayout();
 
   for (SmallVector<ParallelRegion *, 8>::iterator
          i = original_parallel_regions->begin(), 
@@ -203,11 +189,7 @@ WorkitemReplication::ProcessFunction(Function &F)
         for (Value::use_iterator i4 = i3->use_begin(), e4 = i3->use_end();
              i4 != e4; ++i4) {
           // Instructions can only be used by instructions.
-#if defined LLVM_3_2 || defined LLVM_3_3 || defined LLVM_3_4
-          llvm::Instruction *user = cast<Instruction>(*i4);
-#else
           llvm::Instruction *user = cast<Instruction>(i4->getUser());
-#endif
           
           if (find (pr->begin(), pr->end(), user->getParent()) ==
               pr->end()) {
@@ -222,13 +204,14 @@ WorkitemReplication::ProcessFunction(Function &F)
   }
 
   // Then replicate the ParallelRegions.  
-  ValueToValueMapTy *const reference_map = new ValueToValueMapTy[workitem_count - 1];
-  for (int z = 0; z < LocalSizeZ; ++z) {
-    for (int y = 0; y < LocalSizeY; ++y) {
-      for (int x = 0; x < LocalSizeX ; ++x) {
+  ValueToValueMapTy *const reference_map = 
+    new ValueToValueMapTy[workitem_count - 1];
+  for (unsigned z = 0; z < WGLocalSizeZ; ++z) {
+    for (unsigned y = 0; y < WGLocalSizeY; ++y) {
+      for (unsigned x = 0; x < WGLocalSizeX ; ++x) {
               
         int index = 
-          (LocalSizeY * LocalSizeX * z + LocalSizeX * y + x);
+          (WGLocalSizeY * WGLocalSizeX * z + WGLocalSizeX * y + x);
 	  
         if (index == 0)
           continue;
@@ -263,12 +246,12 @@ WorkitemReplication::ProcessFunction(Function &F)
     }
   }  
   
-  for (int z = 0; z < LocalSizeZ; ++z) {
-    for (int y = 0; y < LocalSizeY; ++y) {
-      for (int x = 0; x < LocalSizeX ; ++x) {
+  for (unsigned z = 0; z < WGLocalSizeZ; ++z) {
+    for (unsigned y = 0; y < WGLocalSizeY; ++y) {
+      for (unsigned x = 0; x < WGLocalSizeX ; ++x) {
 	  
         int index = 
-          (LocalSizeY * LocalSizeX * z + LocalSizeX * y + x);
+          (WGLocalSizeY * WGLocalSizeX * z + WGLocalSizeX * y + x);
         
         for (unsigned i = 0, e = parallel_regions[index].size(); i != e; ++i) {
           ParallelRegion *region = parallel_regions[index][i];
@@ -294,12 +277,12 @@ WorkitemReplication::ProcessFunction(Function &F)
   // region copys to the entry BB of the first copy to retain
   // their semantics for branches from outside the parallel
   // region to the beginning of the region copy chain.
-  for (int z = LocalSizeZ - 1; z >= 0; --z) {
-    for (int y = LocalSizeY - 1; y >= 0; --y) {
-      for (int x = LocalSizeX - 1; x >= 0; --x) {
+  for (int z = WGLocalSizeZ - 1; z >= 0; --z) {
+    for (int y = WGLocalSizeY - 1; y >= 0; --y) {
+      for (int x = WGLocalSizeX - 1; x >= 0; --x) {
           
         int index = 
-          (LocalSizeY * LocalSizeX * z + LocalSizeX * y + x);
+          (WGLocalSizeY * WGLocalSizeX * z + WGLocalSizeX * y + x);
 
         if (index == 0)
           continue;
@@ -330,7 +313,7 @@ WorkitemReplication::ProcessFunction(Function &F)
 
   // Initialize local size variables (done at the end to avoid unnecessary
   // replication).
-  K->addLocalSizeInitCode(LocalSizeX, LocalSizeY, LocalSizeZ);
+  K->addLocalSizeInitCode(WGLocalSizeX, WGLocalSizeY, WGLocalSizeZ);
 
   delete [] reference_map;
 
