@@ -15,7 +15,6 @@ int main(int argc, char **argv)
   cl_context ctx;
   cl_command_queue queue;
   cl_device_id did;
-  cl_uint num_krn;
   cl_kernel kernel;
 
   poclu_get_any_device(&ctx, &did, &queue);
@@ -34,9 +33,11 @@ int main(int argc, char **argv)
 
   kernel = clCreateKernel(program, NULL, &err);
   TEST_ASSERT(err == CL_INVALID_VALUE);
+  TEST_ASSERT(kernel == NULL);
 
   kernel = clCreateKernel(program, "nonexistent_kernel", &err);
   TEST_ASSERT(err == CL_INVALID_KERNEL_NAME);
+  TEST_ASSERT(kernel == NULL);
 
   printf("OK\n");
 
